# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import Dict, Sequence, Tuple, Type, Union
import pkg_resources

import google.api_core.client_options as ClientOptions  # type: ignore
from google.api_core import exceptions as core_exceptions  # type: ignore
from google.api_core import gapic_v1  # type: ignore
from google.api_core import retry as retries  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.api_core import operation as gac_operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.cloud.appengine_admin_v1.services.services import pagers
from google.cloud.appengine_admin_v1.types import appengine
from google.cloud.appengine_admin_v1.types import network_settings
from google.cloud.appengine_admin_v1.types import operation as ga_operation
from google.cloud.appengine_admin_v1.types import service
from google.protobuf import empty_pb2  # type: ignore
from .transports.base import ServicesTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import ServicesGrpcAsyncIOTransport
from .client import ServicesClient


class ServicesAsyncClient:
    """Manages services of an application."""

    _client: ServicesClient

    DEFAULT_ENDPOINT = ServicesClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = ServicesClient.DEFAULT_MTLS_ENDPOINT

    common_billing_account_path = staticmethod(
        ServicesClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        ServicesClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(ServicesClient.common_folder_path)
    parse_common_folder_path = staticmethod(ServicesClient.parse_common_folder_path)
    common_organization_path = staticmethod(ServicesClient.common_organization_path)
    parse_common_organization_path = staticmethod(
        ServicesClient.parse_common_organization_path
    )
    common_project_path = staticmethod(ServicesClient.common_project_path)
    parse_common_project_path = staticmethod(ServicesClient.parse_common_project_path)
    common_location_path = staticmethod(ServicesClient.common_location_path)
    parse_common_location_path = staticmethod(ServicesClient.parse_common_location_path)

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            ServicesAsyncClient: The constructed client.
        """
        return ServicesClient.from_service_account_info.__func__(ServicesAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            ServicesAsyncClient: The constructed client.
        """
        return ServicesClient.from_service_account_file.__func__(ServicesAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> ServicesTransport:
        """Returns the transport used by the client instance.

        Returns:
            ServicesTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(ServicesClient).get_transport_class, type(ServicesClient)
    )

    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials = None,
        transport: Union[str, ServicesTransport] = "grpc_asyncio",
        client_options: ClientOptions = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the services client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.ServicesTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = ServicesClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def list_services(
        self,
        request: appengine.ListServicesRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListServicesAsyncPager:
        r"""Lists all the services in the application.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.ListServicesRequest`):
                The request object. Request message for
                `Services.ListServices`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.appengine_admin_v1.services.services.pagers.ListServicesAsyncPager:
                Response message for Services.ListServices.

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        request = appengine.ListServicesRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_services,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListServicesAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_service(
        self,
        request: appengine.GetServiceRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> service.Service:
        r"""Gets the current configuration of the specified
        service.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.GetServiceRequest`):
                The request object. Request message for
                `Services.GetService`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.appengine_admin_v1.types.Service:
                A Service resource is a logical
                component of an application that can
                share state and communicate in a secure
                fashion with other services. For
                example, an application that handles
                customer requests might include separate
                services to handle tasks such as backend
                data analysis or API requests from
                mobile devices. Each service has a
                collection of versions that define a
                specific set of code used to implement
                the functionality of that service.

        """
        # Create or coerce a protobuf request object.
        request = appengine.GetServiceRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_service,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def update_service(
        self,
        request: appengine.UpdateServiceRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Updates the configuration of the specified service.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.UpdateServiceRequest`):
                The request object. Request message for
                `Services.UpdateService`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.appengine_admin_v1.types.Service` A Service resource is a logical component of an application that can share
                   state and communicate in a secure fashion with other
                   services. For example, an application that handles
                   customer requests might include separate services to
                   handle tasks such as backend data analysis or API
                   requests from mobile devices. Each service has a
                   collection of versions that define a specific set of
                   code used to implement the functionality of that
                   service.

        """
        # Create or coerce a protobuf request object.
        request = appengine.UpdateServiceRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_service,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            service.Service,
            metadata_type=ga_operation.OperationMetadataV1,
        )

        # Done; return the response.
        return response

    async def delete_service(
        self,
        request: appengine.DeleteServiceRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Deletes the specified service and all enclosed
        versions.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.DeleteServiceRequest`):
                The request object. Request message for
                `Services.DeleteService`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

                   The JSON representation for Empty is empty JSON
                   object {}.

        """
        # Create or coerce a protobuf request object.
        request = appengine.DeleteServiceRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_service,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=ga_operation.OperationMetadataV1,
        )

        # Done; return the response.
        return response


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-appengine-admin",
        ).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("ServicesAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.appengine.v1", manifest={"AuthorizedDomain",},
)


class AuthorizedDomain(proto.Message):
    r"""A domain that a user has been authorized to administer. To authorize
    use of a domain, verify ownership via `Webmaster
    Central <https://www.google.com/webmasters/verification/home>`__.

    Attributes:
        name (str):
            Full path to the ``AuthorizedDomain`` resource in the API.
            Example: ``apps/myapp/authorizedDomains/example.com``.

            @OutputOnly
        id (str):
            Fully qualified domain name of the domain authorized for
            use. Example: ``example.com``.
    """

    name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.STRING, number=2,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.protobuf import duration_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.appengine.v1",
    manifest={
        "AuthFailAction",
        "LoginRequirement",
        "SecurityLevel",
        "ApiConfigHandler",
        "ErrorHandler",
        "UrlMap",
        "StaticFilesHandler",
        "ScriptHandler",
        "ApiEndpointHandler",
        "HealthCheck",
        "ReadinessCheck",
        "LivenessCheck",
        "Library",
    },
)


class AuthFailAction(proto.Enum):
    r"""Actions to take when the user is not logged in."""
    AUTH_FAIL_ACTION_UNSPECIFIED = 0
    AUTH_FAIL_ACTION_REDIRECT = 1
    AUTH_FAIL_ACTION_UNAUTHORIZED = 2


class LoginRequirement(proto.Enum):
    r"""Methods to restrict access to a URL based on login status."""
    LOGIN_UNSPECIFIED = 0
    LOGIN_OPTIONAL = 1
    LOGIN_ADMIN = 2
    LOGIN_REQUIRED = 3


class SecurityLevel(proto.Enum):
    r"""Methods to enforce security (HTTPS) on a URL."""
    _pb_options = {"allow_alias": True}
    SECURE_UNSPECIFIED = 0
    SECURE_DEFAULT = 0
    SECURE_NEVER = 1
    SECURE_OPTIONAL = 2
    SECURE_ALWAYS = 3


class ApiConfigHandler(proto.Message):
    r"""`Google Cloud
    Endpoints <https://cloud.google.com/appengine/docs/python/endpoints/>`__
    configuration for API handlers.

    Attributes:
        auth_fail_action (google.cloud.appengine_admin_v1.types.AuthFailAction):
            Action to take when users access resources that require
            authentication. Defaults to ``redirect``.
        login (google.cloud.appengine_admin_v1.types.LoginRequirement):
            Level of login required to access this resource. Defaults to
            ``optional``.
        script (str):
            Path to the script from the application root
            directory.
        security_level (google.cloud.appengine_admin_v1.types.SecurityLevel):
            Security (HTTPS) enforcement for this URL.
        url (str):
            URL to serve the endpoint at.
    """

    auth_fail_action = proto.Field(proto.ENUM, number=1, enum="AuthFailAction",)
    login = proto.Field(proto.ENUM, number=2, enum="LoginRequirement",)
    script = proto.Field(proto.STRING, number=3,)
    security_level = proto.Field(proto.ENUM, number=4, enum="SecurityLevel",)
    url = proto.Field(proto.STRING, number=5,)


class ErrorHandler(proto.Message):
    r"""Custom static error page to be served when an error occurs.
    Attributes:
        error_code (google.cloud.appengine_admin_v1.types.ErrorHandler.ErrorCode):
            Error condition this handler applies to.
        static_file (str):
            Static file content to be served for this
            error.
        mime_type (str):
            MIME type of file. Defaults to ``text/html``.
    """

    class ErrorCode(proto.Enum):
        r"""Error codes."""
        _pb_options = {"allow_alias": True}
        ERROR_CODE_UNSPECIFIED = 0
        ERROR_CODE_DEFAULT = 0
        ERROR_CODE_OVER_QUOTA = 1
        ERROR_CODE_DOS_API_DENIAL = 2
        ERROR_CODE_TIMEOUT = 3

    error_code = proto.Field(proto.ENUM, number=1, enum=ErrorCode,)
    static_file = proto.Field(proto.STRING, number=2,)
    mime_type = proto.Field(proto.STRING, number=3,)


class UrlMap(proto.Message):
    r"""URL pattern and description of how the URL should be handled.
    App Engine can handle URLs by executing application code or by
    serving static files uploaded with the version, such as images,
    CSS, or JavaScript.

    Attributes:
        url_regex (str):
            URL prefix. Uses regular expression syntax,
            which means regexp special characters must be
            escaped, but should not contain groupings. All
            URLs that begin with this prefix are handled by
            this handler, using the portion of the URL after
            the prefix as part of the file path.
        static_files (google.cloud.appengine_admin_v1.types.StaticFilesHandler):
            Returns the contents of a file, such as an
            image, as the response.
        script (google.cloud.appengine_admin_v1.types.ScriptHandler):
            Executes a script to handle the requests that match this URL
            pattern. Only the ``auto`` value is supported for Node.js in
            the App Engine standard environment, for example
            ``"script": "auto"``.
        api_endpoint (google.cloud.appengine_admin_v1.types.ApiEndpointHandler):
            Uses API Endpoints to handle requests.
        security_level (google.cloud.appengine_admin_v1.types.SecurityLevel):
            Security (HTTPS) enforcement for this URL.
        login (google.cloud.appengine_admin_v1.types.LoginRequirement):
            Level of login required to access this
            resource. Not supported for Node.js in the App
            Engine standard environment.
        auth_fail_action (google.cloud.appengine_admin_v1.types.AuthFailAction):
            Action to take when users access resources that require
            authentication. Defaults to ``redirect``.
        redirect_http_response_code (google.cloud.appengine_admin_v1.types.UrlMap.RedirectHttpResponseCode):
            ``30x`` code to use when performing redirects for the
            ``secure`` field. Defaults to ``302``.
    """

    class RedirectHttpResponseCode(proto.Enum):
        r"""Redirect codes."""
        REDIRECT_HTTP_RESPONSE_CODE_UNSPECIFIED = 0
        REDIRECT_HTTP_RESPONSE_CODE_301 = 1
        REDIRECT_HTTP_RESPONSE_CODE_302 = 2
        REDIRECT_HTTP_RESPONSE_CODE_303 = 3
        REDIRECT_HTTP_RESPONSE_CODE_307 = 4

    url_regex = proto.Field(proto.STRING, number=1,)
    static_files = proto.Field(
        proto.MESSAGE, number=2, oneof="handler_type", message="StaticFilesHandler",
    )
    script = proto.Field(
        proto.MESSAGE, number=3, oneof="handler_type", message="ScriptHandler",
    )
    api_endpoint = proto.Field(
        proto.MESSAGE, number=4, oneof="handler_type", message="ApiEndpointHandler",
    )
    security_level = proto.Field(proto.ENUM, number=5, enum="SecurityLevel",)
    login = proto.Field(proto.ENUM, number=6, enum="LoginRequirement",)
    auth_fail_action = proto.Field(proto.ENUM, number=7, enum="AuthFailAction",)
    redirect_http_response_code = proto.Field(
        proto.ENUM, number=8, enum=RedirectHttpResponseCode,
    )


class StaticFilesHandler(proto.Message):
    r"""Files served directly to the user for a given URL, such as
    images, CSS stylesheets, or JavaScript source files. Static file
    handlers describe which files in the application directory are
    static files, and which URLs serve them.

    Attributes:
        path (str):
            Path to the static files matched by the URL
            pattern, from the application root directory.
            The path can refer to text matched in groupings
            in the URL pattern.
        upload_path_regex (str):
            Regular expression that matches the file
            paths for all files that should be referenced by
            this handler.
        http_headers (Sequence[google.cloud.appengine_admin_v1.types.StaticFilesHandler.HttpHeadersEntry]):
            HTTP headers to use for all responses from
            these URLs.
        mime_type (str):
            MIME type used to serve all files served by
            this handler.
            Defaults to file-specific MIME types, which are
            derived from each file's filename extension.
        expiration (google.protobuf.duration_pb2.Duration):
            Time a static file served by this handler
            should be cached by web proxies and browsers.
        require_matching_file (bool):
            Whether this handler should match the request
            if the file referenced by the handler does not
            exist.
        application_readable (bool):
            Whether files should also be uploaded as code
            data. By default, files declared in static file
            handlers are uploaded as static data and are
            only served to end users; they cannot be read by
            the application. If enabled, uploads are charged
            against both your code and static data storage
            resource quotas.
    """

    path = proto.Field(proto.STRING, number=1,)
    upload_path_regex = proto.Field(proto.STRING, number=2,)
    http_headers = proto.MapField(proto.STRING, proto.STRING, number=3,)
    mime_type = proto.Field(proto.STRING, number=4,)
    expiration = proto.Field(proto.MESSAGE, number=5, message=duration_pb2.Duration,)
    require_matching_file = proto.Field(proto.BOOL, number=6,)
    application_readable = proto.Field(proto.BOOL, number=7,)


class ScriptHandler(proto.Message):
    r"""Executes a script to handle the request that matches the URL
    pattern.

    Attributes:
        script_path (str):
            Path to the script from the application root
            directory.
    """

    script_path = proto.Field(proto.STRING, number=1,)


class ApiEndpointHandler(proto.Message):
    r"""Uses Google Cloud Endpoints to handle requests.
    Attributes:
        script_path (str):
            Path to the script from the application root
            directory.
    """

    script_path = proto.Field(proto.STRING, number=1,)


class HealthCheck(proto.Message):
    r"""Health checking configuration for VM instances. Unhealthy
    instances are killed and replaced with new instances. Only
    applicable for instances in App Engine flexible environment.

    Attributes:
        disable_health_check (bool):
            Whether to explicitly disable health checks
            for this instance.
        host (str):
            Host header to send when performing an HTTP
            health check. Example: "myapp.appspot.com".
        healthy_threshold (int):
            Number of consecutive successful health
            checks required before receiving traffic.
        unhealthy_threshold (int):
            Number of consecutive failed health checks
            required before removing traffic.
        restart_threshold (int):
            Number of consecutive failed health checks
            required before an instance is restarted.
        check_interval (google.protobuf.duration_pb2.Duration):
            Interval between health checks.
        timeout (google.protobuf.duration_pb2.Duration):
            Time before the health check is considered
            failed.
    """

    disable_health_check = proto.Field(proto.BOOL, number=1,)
    host = proto.Field(proto.STRING, number=2,)
    healthy_threshold = proto.Field(proto.UINT32, number=3,)
    unhealthy_threshold = proto.Field(proto.UINT32, number=4,)
    restart_threshold = proto.Field(proto.UINT32, number=5,)
    check_interval = proto.Field(
        proto.MESSAGE, number=6, message=duration_pb2.Duration,
    )
    timeout = proto.Field(proto.MESSAGE, number=7, message=duration_pb2.Duration,)


class ReadinessCheck(proto.Message):
    r"""Readiness checking configuration for VM instances. Unhealthy
    instances are removed from traffic rotation.

    Attributes:
        path (str):
            The request path.
        host (str):
            Host header to send when performing a HTTP
            Readiness check. Example: "myapp.appspot.com".
        failure_threshold (int):
            Number of consecutive failed checks required
            before removing traffic.
        success_threshold (int):
            Number of consecutive successful checks
            required before receiving traffic.
        check_interval (google.protobuf.duration_pb2.Duration):
            Interval between health checks.
        timeout (google.protobuf.duration_pb2.Duration):
            Time before the check is considered failed.
        app_start_timeout (google.protobuf.duration_pb2.Duration):
            A maximum time limit on application
            initialization, measured from moment the
            application successfully replies to a
            healthcheck until it is ready to serve traffic.
    """

    path = proto.Field(proto.STRING, number=1,)
    host = proto.Field(proto.STRING, number=2,)
    failure_threshold = proto.Field(proto.UINT32, number=3,)
    success_threshold = proto.Field(proto.UINT32, number=4,)
    check_interval = proto.Field(
        proto.MESSAGE, number=5, message=duration_pb2.Duration,
    )
    timeout = proto.Field(proto.MESSAGE, number=6, message=duration_pb2.Duration,)
    app_start_timeout = proto.Field(
        proto.MESSAGE, number=7, message=duration_pb2.Duration,
    )


class LivenessCheck(proto.Message):
    r"""Health checking configuration for VM instances. Unhealthy
    instances are killed and replaced with new instances.

    Attributes:
        path (str):
            The request path.
        host (str):
            Host header to send when performing a HTTP
            Liveness check. Example: "myapp.appspot.com".
        failure_threshold (int):
            Number of consecutive failed checks required
            before considering the VM unhealthy.
        success_threshold (int):
            Number of consecutive successful checks
            required before considering the VM healthy.
        check_interval (google.protobuf.duration_pb2.Duration):
            Interval between health checks.
        timeout (google.protobuf.duration_pb2.Duration):
            Time before the check is considered failed.
        initial_delay (google.protobuf.duration_pb2.Duration):
            The initial delay before starting to execute
            the checks.
    """

    path = proto.Field(proto.STRING, number=1,)
    host = proto.Field(proto.STRING, number=2,)
    failure_threshold = proto.Field(proto.UINT32, number=3,)
    success_threshold = proto.Field(proto.UINT32, number=4,)
    check_interval = proto.Field(
        proto.MESSAGE, number=5, message=duration_pb2.Duration,
    )
    timeout = proto.Field(proto.MESSAGE, number=6, message=duration_pb2.Duration,)
    initial_delay = proto.Field(proto.MESSAGE, number=7, message=duration_pb2.Duration,)


class Library(proto.Message):
    r"""Third-party Python runtime library that is required by the
    application.

    Attributes:
        name (str):
            Name of the library. Example: "django".
        version (str):
            Version of the library to select, or
            "latest".
    """

    name = proto.Field(proto.STRING, number=1,)
    version = proto.Field(proto.STRING, number=2,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.appengine.v1", manifest={"LocationMetadata",},
)


class LocationMetadata(proto.Message):
    r"""Metadata for the given
    [google.cloud.location.Location][google.cloud.location.Location].

    Attributes:
        standard_environment_available (bool):
            App Engine standard environment is available
            in the given location.
            @OutputOnly
        flexible_environment_available (bool):
            App Engine flexible environment is available
            in the given location.
            @OutputOnly
        search_api_available (bool):
            Output only. `Search
            API <https://cloud.google.com/appengine/docs/standard/python/search>`__
            is available in the given location.
    """

    standard_environment_available = proto.Field(proto.BOOL, number=2,)
    flexible_environment_available = proto.Field(proto.BOOL, number=4,)
    search_api_available = proto.Field(proto.BOOL, number=6,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.appengine.v1", manifest={"NetworkSettings",},
)


class NetworkSettings(proto.Message):
    r"""A NetworkSettings resource is a container for ingress
    settings for a version or service.

    Attributes:
        ingress_traffic_allowed (google.cloud.appengine_admin_v1.types.NetworkSettings.IngressTrafficAllowed):
            The ingress settings for version or service.
    """

    class IngressTrafficAllowed(proto.Enum):
        r"""If unspecified, INGRESS_TRAFFIC_ALLOWED_ALL will be used."""
        INGRESS_TRAFFIC_ALLOWED_UNSPECIFIED = 0
        INGRESS_TRAFFIC_ALLOWED_ALL = 1
        INGRESS_TRAFFIC_ALLOWED_INTERNAL_ONLY = 2
        INGRESS_TRAFFIC_ALLOWED_INTERNAL_AND_LB = 3

    ingress_traffic_allowed = proto.Field(
        proto.ENUM, number=1, enum=IngressTrafficAllowed,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(package="google.appengine.v1", manifest={"Instance",},)


class Instance(proto.Message):
    r"""An Instance resource is the computing unit that App Engine
    uses to automatically scale an application.

    Attributes:
        name (str):
            Output only. Full path to the Instance resource in the API.
            Example:
            ``apps/myapp/services/default/versions/v1/instances/instance-1``.
        id (str):
            Output only. Relative name of the instance within the
            version. Example: ``instance-1``.
        app_engine_release (str):
            Output only. App Engine release this instance
            is running on.
        availability (google.cloud.appengine_admin_v1.types.Instance.Availability):
            Output only. Availability of the instance.
        vm_name (str):
            Output only. Name of the virtual machine
            where this instance lives. Only applicable for
            instances in App Engine flexible environment.
        vm_zone_name (str):
            Output only. Zone where the virtual machine
            is located. Only applicable for instances in App
            Engine flexible environment.
        vm_id (str):
            Output only. Virtual machine ID of this
            instance. Only applicable for instances in App
            Engine flexible environment.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time that this instance was
            started.
            @OutputOnly
        requests (int):
            Output only. Number of requests since this
            instance was started.
        errors (int):
            Output only. Number of errors since this
            instance was started.
        qps (float):
            Output only. Average queries per second (QPS)
            over the last minute.
        average_latency (int):
            Output only. Average latency (ms) over the
            last minute.
        memory_usage (int):
            Output only. Total memory in use (bytes).
        vm_status (str):
            Output only. Status of the virtual machine
            where this instance lives. Only applicable for
            instances in App Engine flexible environment.
        vm_debug_enabled (bool):
            Output only. Whether this instance is in
            debug mode. Only applicable for instances in App
            Engine flexible environment.
        vm_ip (str):
            Output only. The IP address of this instance.
            Only applicable for instances in App Engine
            flexible environment.
        vm_liveness (google.cloud.appengine_admin_v1.types.Instance.Liveness.LivenessState):
            Output only. The liveness health check of
            this instance. Only applicable for instances in
            App Engine flexible environment.
    """

    class Availability(proto.Enum):
        r"""Availability of the instance."""
        UNSPECIFIED = 0
        RESIDENT = 1
        DYNAMIC = 2

    class Liveness(proto.Message):
        r"""Wrapper for LivenessState enum.    """

        class LivenessState(proto.Enum):
            r"""Liveness health check status for Flex instances."""
            LIVENESS_STATE_UNSPECIFIED = 0
            UNKNOWN = 1
            HEALTHY = 2
            UNHEALTHY = 3
            DRAINING = 4
            TIMEOUT = 5

    name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.STRING, number=2,)
    app_engine_release = proto.Field(proto.STRING, number=3,)
    availability = proto.Field(proto.ENUM, number=4, enum=Availability,)
    vm_name = proto.Field(proto.STRING, number=5,)
    vm_zone_name = proto.Field(proto.STRING, number=6,)
    vm_id = proto.Field(proto.STRING, number=7,)
    start_time = proto.Field(proto.MESSAGE, number=8, message=timestamp_pb2.Timestamp,)
    requests = proto.Field(proto.INT32, number=9,)
    errors = proto.Field(proto.INT32, number=10,)
    qps = proto.Field(proto.FLOAT, number=11,)
    average_latency = proto.Field(proto.INT32, number=12,)
    memory_usage = proto.Field(proto.INT64, number=13,)
    vm_status = proto.Field(proto.STRING, number=14,)
    vm_debug_enabled = proto.Field(proto.BOOL, number=15,)
    vm_ip = proto.Field(proto.STRING, number=16,)
    vm_liveness = proto.Field(proto.ENUM, number=17, enum=Liveness.LivenessState,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.appengine_admin_v1.types import (
    network_settings as ga_network_settings,
)


__protobuf__ = proto.module(
    package="google.appengine.v1", manifest={"Service", "TrafficSplit",},
)


class Service(proto.Message):
    r"""A Service resource is a logical component of an application
    that can share state and communicate in a secure fashion with
    other services. For example, an application that handles
    customer requests might include separate services to handle
    tasks such as backend data analysis or API requests from mobile
    devices. Each service has a collection of versions that define a
    specific set of code used to implement the functionality of that
    service.

    Attributes:
        name (str):
            Full path to the Service resource in the API. Example:
            ``apps/myapp/services/default``.

            @OutputOnly
        id (str):
            Relative name of the service within the application.
            Example: ``default``.

            @OutputOnly
        split (google.cloud.appengine_admin_v1.types.TrafficSplit):
            Mapping that defines fractional HTTP traffic
            diversion to different versions within the
            service.
        network_settings (google.cloud.appengine_admin_v1.types.NetworkSettings):
            Ingress settings for this service. Will apply
            to all versions.
    """

    name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.STRING, number=2,)
    split = proto.Field(proto.MESSAGE, number=3, message="TrafficSplit",)
    network_settings = proto.Field(
        proto.MESSAGE, number=6, message=ga_network_settings.NetworkSettings,
    )


class TrafficSplit(proto.Message):
    r"""Traffic routing configuration for versions within a single
    service. Traffic splits define how traffic directed to the
    service is assigned to versions.

    Attributes:
        shard_by (google.cloud.appengine_admin_v1.types.TrafficSplit.ShardBy):
            Mechanism used to determine which version a
            request is sent to. The traffic selection
            algorithm will be stable for either type until
            allocations are changed.
        allocations (Sequence[google.cloud.appengine_admin_v1.types.TrafficSplit.AllocationsEntry]):
            Mapping from version IDs within the service to fractional
            (0.000, 1] allocations of traffic for that version. Each
            version can be specified only once, but some versions in the
            service may not have any traffic allocation. Services that
            have traffic allocated cannot be deleted until either the
            service is deleted or their traffic allocation is removed.
            Allocations must sum to 1. Up to two decimal place precision
            is supported for IP-based splits and up to three decimal
            places is supported for cookie-based splits.
    """

    class ShardBy(proto.Enum):
        r"""Available sharding mechanisms."""
        UNSPECIFIED = 0
        COOKIE = 1
        IP = 2
        RANDOM = 3

    shard_by = proto.Field(proto.ENUM, number=1, enum=ShardBy,)
    allocations = proto.MapField(proto.STRING, proto.DOUBLE, number=2,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.appengine_admin_v1.types import app_yaml
from google.cloud.appengine_admin_v1.types import deploy
from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.appengine.v1",
    manifest={
        "InboundServiceType",
        "ServingStatus",
        "Version",
        "EndpointsApiService",
        "AutomaticScaling",
        "BasicScaling",
        "ManualScaling",
        "CpuUtilization",
        "RequestUtilization",
        "DiskUtilization",
        "NetworkUtilization",
        "StandardSchedulerSettings",
        "Network",
        "Volume",
        "Resources",
        "VpcAccessConnector",
        "Entrypoint",
    },
)


class InboundServiceType(proto.Enum):
    r"""Available inbound services."""
    INBOUND_SERVICE_UNSPECIFIED = 0
    INBOUND_SERVICE_MAIL = 1
    INBOUND_SERVICE_MAIL_BOUNCE = 2
    INBOUND_SERVICE_XMPP_ERROR = 3
    INBOUND_SERVICE_XMPP_MESSAGE = 4
    INBOUND_SERVICE_XMPP_SUBSCRIBE = 5
    INBOUND_SERVICE_XMPP_PRESENCE = 6
    INBOUND_SERVICE_CHANNEL_PRESENCE = 7
    INBOUND_SERVICE_WARMUP = 9


class ServingStatus(proto.Enum):
    r"""Run states of a version."""
    SERVING_STATUS_UNSPECIFIED = 0
    SERVING = 1
    STOPPED = 2


class Version(proto.Message):
    r"""A Version resource is a specific set of source code and
    configuration files that are deployed into a service.

    Attributes:
        name (str):
            Full path to the Version resource in the API. Example:
            ``apps/myapp/services/default/versions/v1``.

            @OutputOnly
        id (str):
            Relative name of the version within the service. Example:
            ``v1``. Version names can contain only lowercase letters,
            numbers, or hyphens. Reserved names: "default", "latest",
            and any name with the prefix "ah-".
        automatic_scaling (google.cloud.appengine_admin_v1.types.AutomaticScaling):
            Automatic scaling is based on request rate,
            response latencies, and other application
            metrics. Instances are dynamically created and
            destroyed as needed in order to handle traffic.
        basic_scaling (google.cloud.appengine_admin_v1.types.BasicScaling):
            A service with basic scaling will create an
            instance when the application receives a
            request. The instance will be turned down when
            the app becomes idle. Basic scaling is ideal for
            work that is intermittent or driven by user
            activity.
        manual_scaling (google.cloud.appengine_admin_v1.types.ManualScaling):
            A service with manual scaling runs
            continuously, allowing you to perform complex
            initialization and rely on the state of its
            memory over time. Manually scaled versions are
            sometimes referred to as "backends".
        inbound_services (Sequence[google.cloud.appengine_admin_v1.types.InboundServiceType]):
            Before an application can receive email or
            XMPP messages, the application must be
            configured to enable the service.
        instance_class (str):
            Instance class that is used to run this version. Valid
            values are:

            -  AutomaticScaling: ``F1``, ``F2``, ``F4``, ``F4_1G``
            -  ManualScaling or BasicScaling: ``B1``, ``B2``, ``B4``,
               ``B8``, ``B4_1G``

            Defaults to ``F1`` for AutomaticScaling and ``B1`` for
            ManualScaling or BasicScaling.
        network (google.cloud.appengine_admin_v1.types.Network):
            Extra network settings.
            Only applicable in the App Engine flexible
            environment.
        zones (Sequence[str]):
            The Google Compute Engine zones that are
            supported by this version in the App Engine
            flexible environment. Deprecated.
        resources (google.cloud.appengine_admin_v1.types.Resources):
            Machine resources for this version.
            Only applicable in the App Engine flexible
            environment.
        runtime (str):
            Desired runtime. Example: ``python27``.
        runtime_channel (str):
            The channel of the runtime to use. Only available for some
            runtimes. Defaults to the ``default`` channel.
        threadsafe (bool):
            Whether multiple requests can be dispatched
            to this version at once.
        vm (bool):
            Whether to deploy this version in a container
            on a virtual machine.
        beta_settings (Sequence[google.cloud.appengine_admin_v1.types.Version.BetaSettingsEntry]):
            Metadata settings that are supplied to this
            version to enable beta runtime features.
        env (str):
            App Engine execution environment for this version.

            Defaults to ``standard``.
        serving_status (google.cloud.appengine_admin_v1.types.ServingStatus):
            Current serving status of this version. Only the versions
            with a ``SERVING`` status create instances and can be
            billed.

            ``SERVING_STATUS_UNSPECIFIED`` is an invalid value. Defaults
            to ``SERVING``.
        created_by (str):
            Email address of the user who created this
            version.
            @OutputOnly
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Time that this version was created.
            @OutputOnly
        disk_usage_bytes (int):
            Total size in bytes of all the files that are
            included in this version and currently hosted on
            the App Engine disk.
            @OutputOnly
        runtime_api_version (str):
            The version of the API in the given runtime
            environment. Please see the app.yaml reference
            for valid values at
            https://cloud.google.com/appengine/docs/standard/<language>/config/appref
        runtime_main_executable_path (str):
            The path or name of the app's main
            executable.
        service_account (str):
            The identity that the deployed version will
            run as. Admin API will use the App Engine
            Appspot service account as default if this field
            is neither provided in app.yaml file nor through
            CLI flag.
        handlers (Sequence[google.cloud.appengine_admin_v1.types.UrlMap]):
            An ordered list of URL-matching patterns that should be
            applied to incoming requests. The first matching URL handles
            the request and other request handlers are not attempted.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        error_handlers (Sequence[google.cloud.appengine_admin_v1.types.ErrorHandler]):
            Custom static error pages. Limited to 10KB per page.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        libraries (Sequence[google.cloud.appengine_admin_v1.types.Library]):
            Configuration for third-party Python runtime libraries that
            are required by the application.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        api_config (google.cloud.appengine_admin_v1.types.ApiConfigHandler):
            Serving configuration for `Google Cloud
            Endpoints <https://cloud.google.com/appengine/docs/python/endpoints/>`__.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        env_variables (Sequence[google.cloud.appengine_admin_v1.types.Version.EnvVariablesEntry]):
            Environment variables available to the application.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        build_env_variables (Sequence[google.cloud.appengine_admin_v1.types.Version.BuildEnvVariablesEntry]):
            Environment variables available to the build environment.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        default_expiration (google.protobuf.duration_pb2.Duration):
            Duration that static files should be cached by web proxies
            and browsers. Only applicable if the corresponding
            `StaticFilesHandler <https://cloud.google.com/appengine/docs/admin-api/reference/rest/v1/apps.services.versions#StaticFilesHandler>`__
            does not specify its own expiration time.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        health_check (google.cloud.appengine_admin_v1.types.HealthCheck):
            Configures health checking for instances. Unhealthy
            instances are stopped and replaced with new instances. Only
            applicable in the App Engine flexible environment.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        readiness_check (google.cloud.appengine_admin_v1.types.ReadinessCheck):
            Configures readiness health checking for instances.
            Unhealthy instances are not put into the backend traffic
            rotation.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        liveness_check (google.cloud.appengine_admin_v1.types.LivenessCheck):
            Configures liveness health checking for instances. Unhealthy
            instances are stopped and replaced with new instances

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        nobuild_files_regex (str):
            Files that match this pattern will not be built into this
            version. Only applicable for Go runtimes.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        deployment (google.cloud.appengine_admin_v1.types.Deployment):
            Code and application artifacts that make up this version.

            Only returned in ``GET`` requests if ``view=FULL`` is set.
        version_url (str):
            Serving URL for this version. Example:
            "https://myversion-dot-myservice-dot-
            myapp.appspot.com"
            @OutputOnly
        endpoints_api_service (google.cloud.appengine_admin_v1.types.EndpointsApiService):
            Cloud Endpoints configuration.

            If endpoints_api_service is set, the Cloud Endpoints
            Extensible Service Proxy will be provided to serve the API
            implemented by the app.
        entrypoint (google.cloud.appengine_admin_v1.types.Entrypoint):
            The entrypoint for the application.
        vpc_access_connector (google.cloud.appengine_admin_v1.types.VpcAccessConnector):
            Enables VPC connectivity for standard apps.
    """

    name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.STRING, number=2,)
    automatic_scaling = proto.Field(
        proto.MESSAGE, number=3, oneof="scaling", message="AutomaticScaling",
    )
    basic_scaling = proto.Field(
        proto.MESSAGE, number=4, oneof="scaling", message="BasicScaling",
    )
    manual_scaling = proto.Field(
        proto.MESSAGE, number=5, oneof="scaling", message="ManualScaling",
    )
    inbound_services = proto.RepeatedField(
        proto.ENUM, number=6, enum="InboundServiceType",
    )
    instance_class = proto.Field(proto.STRING, number=7,)
    network = proto.Field(proto.MESSAGE, number=8, message="Network",)
    zones = proto.RepeatedField(proto.STRING, number=118,)
    resources = proto.Field(proto.MESSAGE, number=9, message="Resources",)
    runtime = proto.Field(proto.STRING, number=10,)
    runtime_channel = proto.Field(proto.STRING, number=117,)
    threadsafe = proto.Field(proto.BOOL, number=11,)
    vm = proto.Field(proto.BOOL, number=12,)
    beta_settings = proto.MapField(proto.STRING, proto.STRING, number=13,)
    env = proto.Field(proto.STRING, number=14,)
    serving_status = proto.Field(proto.ENUM, number=15, enum="ServingStatus",)
    created_by = proto.Field(proto.STRING, number=16,)
    create_time = proto.Field(
        proto.MESSAGE, number=17, message=timestamp_pb2.Timestamp,
    )
    disk_usage_bytes = proto.Field(proto.INT64, number=18,)
    runtime_api_version = proto.Field(proto.STRING, number=21,)
    runtime_main_executable_path = proto.Field(proto.STRING, number=22,)
    service_account = proto.Field(proto.STRING, number=127,)
    handlers = proto.RepeatedField(proto.MESSAGE, number=100, message=app_yaml.UrlMap,)
    error_handlers = proto.RepeatedField(
        proto.MESSAGE, number=101, message=app_yaml.ErrorHandler,
    )
    libraries = proto.RepeatedField(
        proto.MESSAGE, number=102, message=app_yaml.Library,
    )
    api_config = proto.Field(
        proto.MESSAGE, number=103, message=app_yaml.ApiConfigHandler,
    )
    env_variables = proto.MapField(proto.STRING, proto.STRING, number=104,)
    build_env_variables = proto.MapField(proto.STRING, proto.STRING, number=125,)
    default_expiration = proto.Field(
        proto.MESSAGE, number=105, message=duration_pb2.Duration,
    )
    health_check = proto.Field(proto.MESSAGE, number=106, message=app_yaml.HealthCheck,)
    readiness_check = proto.Field(
        proto.MESSAGE, number=112, message=app_yaml.ReadinessCheck,
    )
    liveness_check = proto.Field(
        proto.MESSAGE, number=113, message=app_yaml.LivenessCheck,
    )
    nobuild_files_regex = proto.Field(proto.STRING, number=107,)
    deployment = proto.Field(proto.MESSAGE, number=108, message=deploy.Deployment,)
    version_url = proto.Field(proto.STRING, number=109,)
    endpoints_api_service = proto.Field(
        proto.MESSAGE, number=110, message="EndpointsApiService",
    )
    entrypoint = proto.Field(proto.MESSAGE, number=122, message="Entrypoint",)
    vpc_access_connector = proto.Field(
        proto.MESSAGE, number=121, message="VpcAccessConnector",
    )


class EndpointsApiService(proto.Message):
    r"""`Cloud Endpoints <https://cloud.google.com/endpoints>`__
    configuration. The Endpoints API Service provides tooling for
    serving Open API and gRPC endpoints via an NGINX proxy. Only valid
    for App Engine Flexible environment deployments.

    The fields here refer to the name and configuration ID of a
    "service" resource in the `Service Management
    API <https://cloud.google.com/service-management/overview>`__.

    Attributes:
        name (str):
            Endpoints service name which is the name of
            the "service" resource in the Service Management
            API. For example
            "myapi.endpoints.myproject.cloud.goog".
        config_id (str):
            Endpoints service configuration ID as specified by the
            Service Management API. For example "2016-09-19r1".

            By default, the rollout strategy for Endpoints is
            ``RolloutStrategy.FIXED``. This means that Endpoints starts
            up with a particular configuration ID. When a new
            configuration is rolled out, Endpoints must be given the new
            configuration ID. The ``config_id`` field is used to give
            the configuration ID and is required in this case.

            Endpoints also has a rollout strategy called
            ``RolloutStrategy.MANAGED``. When using this, Endpoints
            fetches the latest configuration and does not need the
            configuration ID. In this case, ``config_id`` must be
            omitted.
        rollout_strategy (google.cloud.appengine_admin_v1.types.EndpointsApiService.RolloutStrategy):
            Endpoints rollout strategy. If ``FIXED``, ``config_id`` must
            be specified. If ``MANAGED``, ``config_id`` must be omitted.
        disable_trace_sampling (bool):
            Enable or disable trace sampling. By default,
            this is set to false for enabled.
    """

    class RolloutStrategy(proto.Enum):
        r"""Available rollout strategies."""
        UNSPECIFIED_ROLLOUT_STRATEGY = 0
        FIXED = 1
        MANAGED = 2

    name = proto.Field(proto.STRING, number=1,)
    config_id = proto.Field(proto.STRING, number=2,)
    rollout_strategy = proto.Field(proto.ENUM, number=3, enum=RolloutStrategy,)
    disable_trace_sampling = proto.Field(proto.BOOL, number=4,)


class AutomaticScaling(proto.Message):
    r"""Automatic scaling is based on request rate, response
    latencies, and other application metrics.

    Attributes:
        cool_down_period (google.protobuf.duration_pb2.Duration):
            The time period that the
            `Autoscaler <https://cloud.google.com/compute/docs/autoscaler/>`__
            should wait before it starts collecting information from a
            new instance. This prevents the autoscaler from collecting
            information when the instance is initializing, during which
            the collected usage would not be reliable. Only applicable
            in the App Engine flexible environment.
        cpu_utilization (google.cloud.appengine_admin_v1.types.CpuUtilization):
            Target scaling by CPU usage.
        max_concurrent_requests (int):
            Number of concurrent requests an automatic
            scaling instance can accept before the scheduler
            spawns a new instance.
            Defaults to a runtime-specific value.
        max_idle_instances (int):
            Maximum number of idle instances that should
            be maintained for this version.
        max_total_instances (int):
            Maximum number of instances that should be
            started to handle requests for this version.
        max_pending_latency (google.protobuf.duration_pb2.Duration):
            Maximum amount of time that a request should
            wait in the pending queue before starting a new
            instance to handle it.
        min_idle_instances (int):
            Minimum number of idle instances that should
            be maintained for this version. Only applicable
            for the default version of a service.
        min_total_instances (int):
            Minimum number of running instances that
            should be maintained for this version.
        min_pending_latency (google.protobuf.duration_pb2.Duration):
            Minimum amount of time a request should wait
            in the pending queue before starting a new
            instance to handle it.
        request_utilization (google.cloud.appengine_admin_v1.types.RequestUtilization):
            Target scaling by request utilization.
        disk_utilization (google.cloud.appengine_admin_v1.types.DiskUtilization):
            Target scaling by disk usage.
        network_utilization (google.cloud.appengine_admin_v1.types.NetworkUtilization):
            Target scaling by network usage.
        standard_scheduler_settings (google.cloud.appengine_admin_v1.types.StandardSchedulerSettings):
            Scheduler settings for standard environment.
    """

    cool_down_period = proto.Field(
        proto.MESSAGE, number=1, message=duration_pb2.Duration,
    )
    cpu_utilization = proto.Field(proto.MESSAGE, number=2, message="CpuUtilization",)
    max_concurrent_requests = proto.Field(proto.INT32, number=3,)
    max_idle_instances = proto.Field(proto.INT32, number=4,)
    max_total_instances = proto.Field(proto.INT32, number=5,)
    max_pending_latency = proto.Field(
        proto.MESSAGE, number=6, message=duration_pb2.Duration,
    )
    min_idle_instances = proto.Field(proto.INT32, number=7,)
    min_total_instances = proto.Field(proto.INT32, number=8,)
    min_pending_latency = proto.Field(
        proto.MESSAGE, number=9, message=duration_pb2.Duration,
    )
    request_utilization = proto.Field(
        proto.MESSAGE, number=10, message="RequestUtilization",
    )
    disk_utilization = proto.Field(proto.MESSAGE, number=11, message="DiskUtilization",)
    network_utilization = proto.Field(
        proto.MESSAGE, number=12, message="NetworkUtilization",
    )
    standard_scheduler_settings = proto.Field(
        proto.MESSAGE, number=20, message="StandardSchedulerSettings",
    )


class BasicScaling(proto.Message):
    r"""A service with basic scaling will create an instance when the
    application receives a request. The instance will be turned down
    when the app becomes idle. Basic scaling is ideal for work that
    is intermittent or driven by user activity.

    Attributes:
        idle_timeout (google.protobuf.duration_pb2.Duration):
            Duration of time after the last request that
            an instance must wait before the instance is
            shut down.
        max_instances (int):
            Maximum number of instances to create for
            this version.
    """

    idle_timeout = proto.Field(proto.MESSAGE, number=1, message=duration_pb2.Duration,)
    max_instances = proto.Field(proto.INT32, number=2,)


class ManualScaling(proto.Message):
    r"""A service with manual scaling runs continuously, allowing you
    to perform complex initialization and rely on the state of its
    memory over time.

    Attributes:
        instances (int):
            Number of instances to assign to the service at the start.
            This number can later be altered by using the `Modules
            API <https://cloud.google.com/appengine/docs/python/modules/functions>`__
            ``set_num_instances()`` function.
    """

    instances = proto.Field(proto.INT32, number=1,)


class CpuUtilization(proto.Message):
    r"""Target scaling by CPU usage.
    Attributes:
        aggregation_window_length (google.protobuf.duration_pb2.Duration):
            Period of time over which CPU utilization is
            calculated.
        target_utilization (float):
            Target CPU utilization ratio to maintain when
            scaling. Must be between 0 and 1.
    """

    aggregation_window_length = proto.Field(
        proto.MESSAGE, number=1, message=duration_pb2.Duration,
    )
    target_utilization = proto.Field(proto.DOUBLE, number=2,)


class RequestUtilization(proto.Message):
    r"""Target scaling by request utilization.
    Only applicable in the App Engine flexible environment.

    Attributes:
        target_request_count_per_second (int):
            Target requests per second.
        target_concurrent_requests (int):
            Target number of concurrent requests.
    """

    target_request_count_per_second = proto.Field(proto.INT32, number=1,)
    target_concurrent_requests = proto.Field(proto.INT32, number=2,)


class DiskUtilization(proto.Message):
    r"""Target scaling by disk usage.
    Only applicable in the App Engine flexible environment.

    Attributes:
        target_write_bytes_per_second (int):
            Target bytes written per second.
        target_write_ops_per_second (int):
            Target ops written per second.
        target_read_bytes_per_second (int):
            Target bytes read per second.
        target_read_ops_per_second (int):
            Target ops read per seconds.
    """

    target_write_bytes_per_second = proto.Field(proto.INT32, number=14,)
    target_write_ops_per_second = proto.Field(proto.INT32, number=15,)
    target_read_bytes_per_second = proto.Field(proto.INT32, number=16,)
    target_read_ops_per_second = proto.Field(proto.INT32, number=17,)


class NetworkUtilization(proto.Message):
    r"""Target scaling by network usage.
    Only applicable in the App Engine flexible environment.

    Attributes:
        target_sent_bytes_per_second (int):
            Target bytes sent per second.
        target_sent_packets_per_second (int):
            Target packets sent per second.
        target_received_bytes_per_second (int):
            Target bytes received per second.
        target_received_packets_per_second (int):
            Target packets received per second.
    """

    target_sent_bytes_per_second = proto.Field(proto.INT32, number=1,)
    target_sent_packets_per_second = proto.Field(proto.INT32, number=11,)
    target_received_bytes_per_second = proto.Field(proto.INT32, number=12,)
    target_received_packets_per_second = proto.Field(proto.INT32, number=13,)


class StandardSchedulerSettings(proto.Message):
    r"""Scheduler settings for standard environment.
    Attributes:
        target_cpu_utilization (float):
            Target CPU utilization ratio to maintain when
            scaling.
        target_throughput_utilization (float):
            Target throughput utilization ratio to
            maintain when scaling
        min_instances (int):
            Minimum number of instances to run for this version. Set to
            zero to disable ``min_instances`` configuration.
        max_instances (int):
            Maximum number of instances to run for this version. Set to
            zero to disable ``max_instances`` configuration.
    """

    target_cpu_utilization = proto.Field(proto.DOUBLE, number=1,)
    target_throughput_utilization = proto.Field(proto.DOUBLE, number=2,)
    min_instances = proto.Field(proto.INT32, number=3,)
    max_instances = proto.Field(proto.INT32, number=4,)


class Network(proto.Message):
    r"""Extra network settings.
    Only applicable in the App Engine flexible environment.

    Attributes:
        forwarded_ports (Sequence[str]):
            List of ports, or port pairs, to forward from
            the virtual machine to the application
            container. Only applicable in the App Engine
            flexible environment.
        instance_tag (str):
            Tag to apply to the instance during creation.
            Only applicable in the App Engine flexible
            environment.
        name (str):
            Google Compute Engine network where the virtual machines are
            created. Specify the short name, not the resource path.

            Defaults to ``default``.
        subnetwork_name (str):
            Google Cloud Platform sub-network where the virtual machines
            are created. Specify the short name, not the resource path.

            If a subnetwork name is specified, a network name will also
            be required unless it is for the default network.

            -  If the network that the instance is being created in is a
               Legacy network, then the IP address is allocated from the
               IPv4Range.
            -  If the network that the instance is being created in is
               an auto Subnet Mode Network, then only network name
               should be specified (not the subnetwork_name) and the IP
               address is created from the IPCidrRange of the subnetwork
               that exists in that zone for that network.
            -  If the network that the instance is being created in is a
               custom Subnet Mode Network, then the subnetwork_name must
               be specified and the IP address is created from the
               IPCidrRange of the subnetwork.

            If specified, the subnetwork must exist in the same region
            as the App Engine flexible environment application.
        session_affinity (bool):
            Enable session affinity.
            Only applicable in the App Engine flexible
            environment.
    """

    forwarded_ports = proto.RepeatedField(proto.STRING, number=1,)
    instance_tag = proto.Field(proto.STRING, number=2,)
    name = proto.Field(proto.STRING, number=3,)
    subnetwork_name = proto.Field(proto.STRING, number=4,)
    session_affinity = proto.Field(proto.BOOL, number=5,)


class Volume(proto.Message):
    r"""Volumes mounted within the app container.
    Only applicable in the App Engine flexible environment.

    Attributes:
        name (str):
            Unique name for the volume.
        volume_type (str):
            Underlying volume type, e.g. 'tmpfs'.
        size_gb (float):
            Volume size in gigabytes.
    """

    name = proto.Field(proto.STRING, number=1,)
    volume_type = proto.Field(proto.STRING, number=2,)
    size_gb = proto.Field(proto.DOUBLE, number=3,)


class Resources(proto.Message):
    r"""Machine resources for a version.
    Attributes:
        cpu (float):
            Number of CPU cores needed.
        disk_gb (float):
            Disk size (GB) needed.
        memory_gb (float):
            Memory (GB) needed.
        volumes (Sequence[google.cloud.appengine_admin_v1.types.Volume]):
            User specified volumes.
        kms_key_reference (str):
            The name of the encryption key that is stored
            in Google Cloud KMS. Only should be used by
            Cloud Composer to encrypt the vm disk
    """

    cpu = proto.Field(proto.DOUBLE, number=1,)
    disk_gb = proto.Field(proto.DOUBLE, number=2,)
    memory_gb = proto.Field(proto.DOUBLE, number=3,)
    volumes = proto.RepeatedField(proto.MESSAGE, number=4, message="Volume",)
    kms_key_reference = proto.Field(proto.STRING, number=5,)


class VpcAccessConnector(proto.Message):
    r"""VPC access connector specification.
    Attributes:
        name (str):
            Full Serverless VPC Access Connector name
            e.g. /projects/my-project/locations/us-
            central1/connectors/c1.
    """

    name = proto.Field(proto.STRING, number=1,)


class Entrypoint(proto.Message):
    r"""The entrypoint for the application.
    Attributes:
        shell (str):
            The format should be a shell command that can be fed to
            ``bash -c``.
    """

    shell = proto.Field(proto.STRING, number=1, oneof="command",)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.appengine.v1",
    manifest={"DomainMapping", "SslSettings", "ResourceRecord",},
)


class DomainMapping(proto.Message):
    r"""A domain serving an App Engine application.
    Attributes:
        name (str):
            Full path to the ``DomainMapping`` resource in the API.
            Example: ``apps/myapp/domainMapping/example.com``.

            @OutputOnly
        id (str):
            Relative name of the domain serving the application.
            Example: ``example.com``.
        ssl_settings (google.cloud.appengine_admin_v1.types.SslSettings):
            SSL configuration for this domain. If
            unconfigured, this domain will not serve with
            SSL.
        resource_records (Sequence[google.cloud.appengine_admin_v1.types.ResourceRecord]):
            The resource records required to configure
            this domain mapping. These records must be added
            to the domain's DNS configuration in order to
            serve the application via this domain mapping.
            @OutputOnly
    """

    name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.STRING, number=2,)
    ssl_settings = proto.Field(proto.MESSAGE, number=3, message="SslSettings",)
    resource_records = proto.RepeatedField(
        proto.MESSAGE, number=4, message="ResourceRecord",
    )


class SslSettings(proto.Message):
    r"""SSL configuration for a ``DomainMapping`` resource.
    Attributes:
        certificate_id (str):
            ID of the ``AuthorizedCertificate`` resource configuring SSL
            for the application. Clearing this field will remove SSL
            support.

            By default, a managed certificate is automatically created
            for every domain mapping. To omit SSL support or to
            configure SSL manually, specify ``SslManagementType.MANUAL``
            on a ``CREATE`` or ``UPDATE`` request. You must be
            authorized to administer the ``AuthorizedCertificate``
            resource to manually map it to a ``DomainMapping`` resource.
            Example: ``12345``.
        ssl_management_type (google.cloud.appengine_admin_v1.types.SslSettings.SslManagementType):
            SSL management type for this domain. If ``AUTOMATIC``, a
            managed certificate is automatically provisioned. If
            ``MANUAL``, ``certificate_id`` must be manually specified in
            order to configure SSL for this domain.
        pending_managed_certificate_id (str):
            ID of the managed ``AuthorizedCertificate`` resource
            currently being provisioned, if applicable. Until the new
            managed certificate has been successfully provisioned, the
            previous SSL state will be preserved. Once the provisioning
            process completes, the ``certificate_id`` field will reflect
            the new managed certificate and this field will be left
            empty. To remove SSL support while there is still a pending
            managed certificate, clear the ``certificate_id`` field with
            an ``UpdateDomainMappingRequest``.

            @OutputOnly
    """

    class SslManagementType(proto.Enum):
        r"""The SSL management type for this domain."""
        SSL_MANAGEMENT_TYPE_UNSPECIFIED = 0
        AUTOMATIC = 1
        MANUAL = 2

    certificate_id = proto.Field(proto.STRING, number=1,)
    ssl_management_type = proto.Field(proto.ENUM, number=3, enum=SslManagementType,)
    pending_managed_certificate_id = proto.Field(proto.STRING, number=4,)


class ResourceRecord(proto.Message):
    r"""A DNS resource record.
    Attributes:
        name (str):
            Relative name of the object affected by this record. Only
            applicable for ``CNAME`` records. Example: 'www'.
        rrdata (str):
            Data for this record. Values vary by record
            type, as defined in RFC 1035 (section 5) and RFC
            1034 (section 3.6.1).
        type_ (google.cloud.appengine_admin_v1.types.ResourceRecord.RecordType):
            Resource record type. Example: ``AAAA``.
    """

    class RecordType(proto.Enum):
        r"""A resource record type."""
        RECORD_TYPE_UNSPECIFIED = 0
        A = 1
        AAAA = 2
        CNAME = 3

    name = proto.Field(proto.STRING, number=1,)
    rrdata = proto.Field(proto.STRING, number=2,)
    type_ = proto.Field(proto.ENUM, number=3, enum=RecordType,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import Dict, Sequence, Tuple, Type, Union
import pkg_resources

import google.api_core.client_options as ClientOptions  # type: ignore
from google.api_core import exceptions as core_exceptions  # type: ignore
from google.api_core import gapic_v1  # type: ignore
from google.api_core import retry as retries  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.appengine_admin_v1.services.firewall import pagers
from google.cloud.appengine_admin_v1.types import appengine
from google.cloud.appengine_admin_v1.types import firewall
from .transports.base import FirewallTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import FirewallGrpcAsyncIOTransport
from .client import FirewallClient


class FirewallAsyncClient:
    """Firewall resources are used to define a collection of access
    control rules for an Application. Each rule is defined with a
    position which specifies the rule's order in the sequence of
    rules, an IP range to be matched against requests, and an action
    to take upon matching requests.
    Every request is evaluated against the Firewall rules in
    priority order. Processesing stops at the first rule which
    matches the request's IP address. A final rule always specifies
    an action that applies to all remaining IP addresses. The
    default final rule for a newly-created application will be set
    to "allow" if not otherwise specified by the user.
    """

    _client: FirewallClient

    DEFAULT_ENDPOINT = FirewallClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = FirewallClient.DEFAULT_MTLS_ENDPOINT

    common_billing_account_path = staticmethod(
        FirewallClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        FirewallClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(FirewallClient.common_folder_path)
    parse_common_folder_path = staticmethod(FirewallClient.parse_common_folder_path)
    common_organization_path = staticmethod(FirewallClient.common_organization_path)
    parse_common_organization_path = staticmethod(
        FirewallClient.parse_common_organization_path
    )
    common_project_path = staticmethod(FirewallClient.common_project_path)
    parse_common_project_path = staticmethod(FirewallClient.parse_common_project_path)
    common_location_path = staticmethod(FirewallClient.common_location_path)
    parse_common_location_path = staticmethod(FirewallClient.parse_common_location_path)

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            FirewallAsyncClient: The constructed client.
        """
        return FirewallClient.from_service_account_info.__func__(FirewallAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            FirewallAsyncClient: The constructed client.
        """
        return FirewallClient.from_service_account_file.__func__(FirewallAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> FirewallTransport:
        """Returns the transport used by the client instance.

        Returns:
            FirewallTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(FirewallClient).get_transport_class, type(FirewallClient)
    )

    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials = None,
        transport: Union[str, FirewallTransport] = "grpc_asyncio",
        client_options: ClientOptions = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the firewall client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.FirewallTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = FirewallClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def list_ingress_rules(
        self,
        request: appengine.ListIngressRulesRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListIngressRulesAsyncPager:
        r"""Lists the firewall rules of an application.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.ListIngressRulesRequest`):
                The request object. Request message for
                `Firewall.ListIngressRules`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.appengine_admin_v1.services.firewall.pagers.ListIngressRulesAsyncPager:
                Response message for Firewall.ListIngressRules.

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        request = appengine.ListIngressRulesRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_ingress_rules,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListIngressRulesAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def batch_update_ingress_rules(
        self,
        request: appengine.BatchUpdateIngressRulesRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> appengine.BatchUpdateIngressRulesResponse:
        r"""Replaces the entire firewall ruleset in one bulk operation. This
        overrides and replaces the rules of an existing firewall with
        the new rules.

        If the final rule does not match traffic with the '*' wildcard
        IP range, then an "allow all" rule is explicitly added to the
        end of the list.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.BatchUpdateIngressRulesRequest`):
                The request object. Request message for
                `Firewall.BatchUpdateIngressRules`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.appengine_admin_v1.types.BatchUpdateIngressRulesResponse:
                Response message for Firewall.UpdateAllIngressRules.
        """
        # Create or coerce a protobuf request object.
        request = appengine.BatchUpdateIngressRulesRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.batch_update_ingress_rules,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def create_ingress_rule(
        self,
        request: appengine.CreateIngressRuleRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> firewall.FirewallRule:
        r"""Creates a firewall rule for the application.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.CreateIngressRuleRequest`):
                The request object. Request message for
                `Firewall.CreateIngressRule`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.appengine_admin_v1.types.FirewallRule:
                A single firewall rule that is
                evaluated against incoming traffic and
                provides an action to take on matched
                requests.

        """
        # Create or coerce a protobuf request object.
        request = appengine.CreateIngressRuleRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_ingress_rule,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def get_ingress_rule(
        self,
        request: appengine.GetIngressRuleRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> firewall.FirewallRule:
        r"""Gets the specified firewall rule.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.GetIngressRuleRequest`):
                The request object. Request message for
                `Firewall.GetIngressRule`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.appengine_admin_v1.types.FirewallRule:
                A single firewall rule that is
                evaluated against incoming traffic and
                provides an action to take on matched
                requests.

        """
        # Create or coerce a protobuf request object.
        request = appengine.GetIngressRuleRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_ingress_rule,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def update_ingress_rule(
        self,
        request: appengine.UpdateIngressRuleRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> firewall.FirewallRule:
        r"""Updates the specified firewall rule.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.UpdateIngressRuleRequest`):
                The request object. Request message for
                `Firewall.UpdateIngressRule`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.appengine_admin_v1.types.FirewallRule:
                A single firewall rule that is
                evaluated against incoming traffic and
                provides an action to take on matched
                requests.

        """
        # Create or coerce a protobuf request object.
        request = appengine.UpdateIngressRuleRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_ingress_rule,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def delete_ingress_rule(
        self,
        request: appengine.DeleteIngressRuleRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Deletes the specified firewall rule.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.DeleteIngressRuleRequest`):
                The request object. Request message for
                `Firewall.DeleteIngressRule`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        request = appengine.DeleteIngressRuleRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_ingress_rule,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request, retry=retry, timeout=timeout, metadata=metadata,
        )


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-appengine-admin",
        ).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("FirewallAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(package="google.appengine.v1", manifest={"FirewallRule",},)


class FirewallRule(proto.Message):
    r"""A single firewall rule that is evaluated against incoming
    traffic and provides an action to take on matched requests.

    Attributes:
        priority (int):
            A positive integer between [1, Int32.MaxValue-1] that
            defines the order of rule evaluation. Rules with the lowest
            priority are evaluated first.

            A default rule at priority Int32.MaxValue matches all IPv4
            and IPv6 traffic when no previous rule matches. Only the
            action of this rule can be modified by the user.
        action (google.cloud.appengine_admin_v1.types.FirewallRule.Action):
            The action to take on matched requests.
        source_range (str):
            IP address or range, defined using CIDR notation, of
            requests that this rule applies to. You can use the wildcard
            character "*" to match all IPs equivalent to "0/0" and
            "::/0" together. Examples: ``192.168.1.1`` or
            ``192.168.0.0/16`` or ``2001:db8::/32`` or
            ``2001:0db8:0000:0042:0000:8a2e:0370:7334``.

            .. raw:: html

                <p>Truncation will be silently performed on addresses which are not
                properly truncated. For example, `1.2.3.4/24` is accepted as the same
                address as `1.2.3.0/24`. Similarly, for IPv6, `2001:db8::1/32` is accepted
                as the same address as `2001:db8::/32`.
        description (str):
            An optional string description of this rule.
            This field has a maximum length of 100
            characters.
    """

    class Action(proto.Enum):
        r"""Available actions to take on matching requests."""
        UNSPECIFIED_ACTION = 0
        ALLOW = 1
        DENY = 2

    priority = proto.Field(proto.INT32, number=1,)
    action = proto.Field(proto.ENUM, number=2, enum=Action,)
    source_range = proto.Field(proto.STRING, number=3,)
    description = proto.Field(proto.STRING, number=4,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from google.cloud.appengine_admin_v1.services.applications.client import (
    ApplicationsClient,
)
from google.cloud.appengine_admin_v1.services.applications.async_client import (
    ApplicationsAsyncClient,
)
from google.cloud.appengine_admin_v1.services.authorized_certificates.client import (
    AuthorizedCertificatesClient,
)
from google.cloud.appengine_admin_v1.services.authorized_certificates.async_client import (
    AuthorizedCertificatesAsyncClient,
)
from google.cloud.appengine_admin_v1.services.authorized_domains.client import (
    AuthorizedDomainsClient,
)
from google.cloud.appengine_admin_v1.services.authorized_domains.async_client import (
    AuthorizedDomainsAsyncClient,
)
from google.cloud.appengine_admin_v1.services.domain_mappings.client import (
    DomainMappingsClient,
)
from google.cloud.appengine_admin_v1.services.domain_mappings.async_client import (
    DomainMappingsAsyncClient,
)
from google.cloud.appengine_admin_v1.services.firewall.client import FirewallClient
from google.cloud.appengine_admin_v1.services.firewall.async_client import (
    FirewallAsyncClient,
)
from google.cloud.appengine_admin_v1.services.instances.client import InstancesClient
from google.cloud.appengine_admin_v1.services.instances.async_client import (
    InstancesAsyncClient,
)
from google.cloud.appengine_admin_v1.services.services.client import ServicesClient
from google.cloud.appengine_admin_v1.services.services.async_client import (
    ServicesAsyncClient,
)
from google.cloud.appengine_admin_v1.services.versions.client import VersionsClient
from google.cloud.appengine_admin_v1.services.versions.async_client import (
    VersionsAsyncClient,
)

from google.cloud.appengine_admin_v1.types.app_yaml import ApiConfigHandler
from google.cloud.appengine_admin_v1.types.app_yaml import ApiEndpointHandler
from google.cloud.appengine_admin_v1.types.app_yaml import ErrorHandler
from google.cloud.appengine_admin_v1.types.app_yaml import HealthCheck
from google.cloud.appengine_admin_v1.types.app_yaml import Library
from google.cloud.appengine_admin_v1.types.app_yaml import LivenessCheck
from google.cloud.appengine_admin_v1.types.app_yaml import ReadinessCheck
from google.cloud.appengine_admin_v1.types.app_yaml import ScriptHandler
from google.cloud.appengine_admin_v1.types.app_yaml import StaticFilesHandler
from google.cloud.appengine_admin_v1.types.app_yaml import UrlMap
from google.cloud.appengine_admin_v1.types.app_yaml import AuthFailAction
from google.cloud.appengine_admin_v1.types.app_yaml import LoginRequirement
from google.cloud.appengine_admin_v1.types.app_yaml import SecurityLevel
from google.cloud.appengine_admin_v1.types.appengine import (
    BatchUpdateIngressRulesRequest,
)
from google.cloud.appengine_admin_v1.types.appengine import (
    BatchUpdateIngressRulesResponse,
)
from google.cloud.appengine_admin_v1.types.appengine import CreateApplicationRequest
from google.cloud.appengine_admin_v1.types.appengine import (
    CreateAuthorizedCertificateRequest,
)
from google.cloud.appengine_admin_v1.types.appengine import CreateDomainMappingRequest
from google.cloud.appengine_admin_v1.types.appengine import CreateIngressRuleRequest
from google.cloud.appengine_admin_v1.types.appengine import CreateVersionRequest
from google.cloud.appengine_admin_v1.types.appengine import DebugInstanceRequest
from google.cloud.appengine_admin_v1.types.appengine import (
    DeleteAuthorizedCertificateRequest,
)
from google.cloud.appengine_admin_v1.types.appengine import DeleteDomainMappingRequest
from google.cloud.appengine_admin_v1.types.appengine import DeleteIngressRuleRequest
from google.cloud.appengine_admin_v1.types.appengine import DeleteInstanceRequest
from google.cloud.appengine_admin_v1.types.appengine import DeleteServiceRequest
from google.cloud.appengine_admin_v1.types.appengine import DeleteVersionRequest
from google.cloud.appengine_admin_v1.types.appengine import GetApplicationRequest
from google.cloud.appengine_admin_v1.types.appengine import (
    GetAuthorizedCertificateRequest,
)
from google.cloud.appengine_admin_v1.types.appengine import GetDomainMappingRequest
from google.cloud.appengine_admin_v1.types.appengine import GetIngressRuleRequest
from google.cloud.appengine_admin_v1.types.appengine import GetInstanceRequest
from google.cloud.appengine_admin_v1.types.appengine import GetServiceRequest
from google.cloud.appengine_admin_v1.types.appengine import GetVersionRequest
from google.cloud.appengine_admin_v1.types.appengine import (
    ListAuthorizedCertificatesRequest,
)
from google.cloud.appengine_admin_v1.types.appengine import (
    ListAuthorizedCertificatesResponse,
)
from google.cloud.appengine_admin_v1.types.appengine import ListAuthorizedDomainsRequest
from google.cloud.appengine_admin_v1.types.appengine import (
    ListAuthorizedDomainsResponse,
)
from google.cloud.appengine_admin_v1.types.appengine import ListDomainMappingsRequest
from google.cloud.appengine_admin_v1.types.appengine import ListDomainMappingsResponse
from google.cloud.appengine_admin_v1.types.appengine import ListIngressRulesRequest
from google.cloud.appengine_admin_v1.types.appengine import ListIngressRulesResponse
from google.cloud.appengine_admin_v1.types.appengine import ListInstancesRequest
from google.cloud.appengine_admin_v1.types.appengine import ListInstancesResponse
from google.cloud.appengine_admin_v1.types.appengine import ListServicesRequest
from google.cloud.appengine_admin_v1.types.appengine import ListServicesResponse
from google.cloud.appengine_admin_v1.types.appengine import ListVersionsRequest
from google.cloud.appengine_admin_v1.types.appengine import ListVersionsResponse
from google.cloud.appengine_admin_v1.types.appengine import RepairApplicationRequest
from google.cloud.appengine_admin_v1.types.appengine import UpdateApplicationRequest
from google.cloud.appengine_admin_v1.types.appengine import (
    UpdateAuthorizedCertificateRequest,
)
from google.cloud.appengine_admin_v1.types.appengine import UpdateDomainMappingRequest
from google.cloud.appengine_admin_v1.types.appengine import UpdateIngressRuleRequest
from google.cloud.appengine_admin_v1.types.appengine import UpdateServiceRequest
from google.cloud.appengine_admin_v1.types.appengine import UpdateVersionRequest
from google.cloud.appengine_admin_v1.types.appengine import AuthorizedCertificateView
from google.cloud.appengine_admin_v1.types.appengine import DomainOverrideStrategy
from google.cloud.appengine_admin_v1.types.appengine import VersionView
from google.cloud.appengine_admin_v1.types.application import Application
from google.cloud.appengine_admin_v1.types.application import UrlDispatchRule
from google.cloud.appengine_admin_v1.types.audit_data import AuditData
from google.cloud.appengine_admin_v1.types.audit_data import CreateVersionMethod
from google.cloud.appengine_admin_v1.types.audit_data import UpdateServiceMethod
from google.cloud.appengine_admin_v1.types.certificate import AuthorizedCertificate
from google.cloud.appengine_admin_v1.types.certificate import CertificateRawData
from google.cloud.appengine_admin_v1.types.certificate import ManagedCertificate
from google.cloud.appengine_admin_v1.types.certificate import ManagementStatus
from google.cloud.appengine_admin_v1.types.deploy import CloudBuildOptions
from google.cloud.appengine_admin_v1.types.deploy import ContainerInfo
from google.cloud.appengine_admin_v1.types.deploy import Deployment
from google.cloud.appengine_admin_v1.types.deploy import FileInfo
from google.cloud.appengine_admin_v1.types.deploy import ZipInfo
from google.cloud.appengine_admin_v1.types.domain import AuthorizedDomain
from google.cloud.appengine_admin_v1.types.domain_mapping import DomainMapping
from google.cloud.appengine_admin_v1.types.domain_mapping import ResourceRecord
from google.cloud.appengine_admin_v1.types.domain_mapping import SslSettings
from google.cloud.appengine_admin_v1.types.firewall import FirewallRule
from google.cloud.appengine_admin_v1.types.instance import Instance
from google.cloud.appengine_admin_v1.types.location import LocationMetadata
from google.cloud.appengine_admin_v1.types.network_settings import NetworkSettings
from google.cloud.appengine_admin_v1.types.operation import CreateVersionMetadataV1
from google.cloud.appengine_admin_v1.types.operation import OperationMetadataV1
from google.cloud.appengine_admin_v1.types.service import Service
from google.cloud.appengine_admin_v1.types.service import TrafficSplit
from google.cloud.appengine_admin_v1.types.version import AutomaticScaling
from google.cloud.appengine_admin_v1.types.version import BasicScaling
from google.cloud.appengine_admin_v1.types.version import CpuUtilization
from google.cloud.appengine_admin_v1.types.version import DiskUtilization
from google.cloud.appengine_admin_v1.types.version import EndpointsApiService
from google.cloud.appengine_admin_v1.types.version import Entrypoint
from google.cloud.appengine_admin_v1.types.version import ManualScaling
from google.cloud.appengine_admin_v1.types.version import Network
from google.cloud.appengine_admin_v1.types.version import NetworkUtilization
from google.cloud.appengine_admin_v1.types.version import RequestUtilization
from google.cloud.appengine_admin_v1.types.version import Resources
from google.cloud.appengine_admin_v1.types.version import StandardSchedulerSettings
from google.cloud.appengine_admin_v1.types.version import Version
from google.cloud.appengine_admin_v1.types.version import Volume
from google.cloud.appengine_admin_v1.types.version import VpcAccessConnector
from google.cloud.appengine_admin_v1.types.version import InboundServiceType
from google.cloud.appengine_admin_v1.types.version import ServingStatus

__all__ = (
    "ApplicationsClient",
    "ApplicationsAsyncClient",
    "AuthorizedCertificatesClient",
    "AuthorizedCertificatesAsyncClient",
    "AuthorizedDomainsClient",
    "AuthorizedDomainsAsyncClient",
    "DomainMappingsClient",
    "DomainMappingsAsyncClient",
    "FirewallClient",
    "FirewallAsyncClient",
    "InstancesClient",
    "InstancesAsyncClient",
    "ServicesClient",
    "ServicesAsyncClient",
    "VersionsClient",
    "VersionsAsyncClient",
    "ApiConfigHandler",
    "ApiEndpointHandler",
    "ErrorHandler",
    "HealthCheck",
    "Library",
    "LivenessCheck",
    "ReadinessCheck",
    "ScriptHandler",
    "StaticFilesHandler",
    "UrlMap",
    "AuthFailAction",
    "LoginRequirement",
    "SecurityLevel",
    "BatchUpdateIngressRulesRequest",
    "BatchUpdateIngressRulesResponse",
    "CreateApplicationRequest",
    "CreateAuthorizedCertificateRequest",
    "CreateDomainMappingRequest",
    "CreateIngressRuleRequest",
    "CreateVersionRequest",
    "DebugInstanceRequest",
    "DeleteAuthorizedCertificateRequest",
    "DeleteDomainMappingRequest",
    "DeleteIngressRuleRequest",
    "DeleteInstanceRequest",
    "DeleteServiceRequest",
    "DeleteVersionRequest",
    "GetApplicationRequest",
    "GetAuthorizedCertificateRequest",
    "GetDomainMappingRequest",
    "GetIngressRuleRequest",
    "GetInstanceRequest",
    "GetServiceRequest",
    "GetVersionRequest",
    "ListAuthorizedCertificatesRequest",
    "ListAuthorizedCertificatesResponse",
    "ListAuthorizedDomainsRequest",
    "ListAuthorizedDomainsResponse",
    "ListDomainMappingsRequest",
    "ListDomainMappingsResponse",
    "ListIngressRulesRequest",
    "ListIngressRulesResponse",
    "ListInstancesRequest",
    "ListInstancesResponse",
    "ListServicesRequest",
    "ListServicesResponse",
    "ListVersionsRequest",
    "ListVersionsResponse",
    "RepairApplicationRequest",
    "UpdateApplicationRequest",
    "UpdateAuthorizedCertificateRequest",
    "UpdateDomainMappingRequest",
    "UpdateIngressRuleRequest",
    "UpdateServiceRequest",
    "UpdateVersionRequest",
    "AuthorizedCertificateView",
    "DomainOverrideStrategy",
    "VersionView",
    "Application",
    "UrlDispatchRule",
    "AuditData",
    "CreateVersionMethod",
    "UpdateServiceMethod",
    "AuthorizedCertificate",
    "CertificateRawData",
    "ManagedCertificate",
    "ManagementStatus",
    "CloudBuildOptions",
    "ContainerInfo",
    "Deployment",
    "FileInfo",
    "ZipInfo",
    "AuthorizedDomain",
    "DomainMapping",
    "ResourceRecord",
    "SslSettings",
    "FirewallRule",
    "Instance",
    "LocationMetadata",
    "NetworkSettings",
    "CreateVersionMetadataV1",
    "OperationMetadataV1",
    "Service",
    "TrafficSplit",
    "AutomaticScaling",
    "BasicScaling",
    "CpuUtilization",
    "DiskUtilization",
    "EndpointsApiService",
    "Entrypoint",
    "ManualScaling",
    "Network",
    "NetworkUtilization",
    "RequestUtilization",
    "Resources",
    "StandardSchedulerSettings",
    "Version",
    "Volume",
    "VpcAccessConnector",
    "InboundServiceType",
    "ServingStatus",
)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .services.applications import ApplicationsClient
from .services.applications import ApplicationsAsyncClient
from .services.authorized_certificates import AuthorizedCertificatesClient
from .services.authorized_certificates import AuthorizedCertificatesAsyncClient
from .services.authorized_domains import AuthorizedDomainsClient
from .services.authorized_domains import AuthorizedDomainsAsyncClient
from .services.domain_mappings import DomainMappingsClient
from .services.domain_mappings import DomainMappingsAsyncClient
from .services.firewall import FirewallClient
from .services.firewall import FirewallAsyncClient
from .services.instances import InstancesClient
from .services.instances import InstancesAsyncClient
from .services.services import ServicesClient
from .services.services import ServicesAsyncClient
from .services.versions import VersionsClient
from .services.versions import VersionsAsyncClient

from .types.app_yaml import ApiConfigHandler
from .types.app_yaml import ApiEndpointHandler
from .types.app_yaml import ErrorHandler
from .types.app_yaml import HealthCheck
from .types.app_yaml import Library
from .types.app_yaml import LivenessCheck
from .types.app_yaml import ReadinessCheck
from .types.app_yaml import ScriptHandler
from .types.app_yaml import StaticFilesHandler
from .types.app_yaml import UrlMap
from .types.app_yaml import AuthFailAction
from .types.app_yaml import LoginRequirement
from .types.app_yaml import SecurityLevel
from .types.appengine import BatchUpdateIngressRulesRequest
from .types.appengine import BatchUpdateIngressRulesResponse
from .types.appengine import CreateApplicationRequest
from .types.appengine import CreateAuthorizedCertificateRequest
from .types.appengine import CreateDomainMappingRequest
from .types.appengine import CreateIngressRuleRequest
from .types.appengine import CreateVersionRequest
from .types.appengine import DebugInstanceRequest
from .types.appengine import DeleteAuthorizedCertificateRequest
from .types.appengine import DeleteDomainMappingRequest
from .types.appengine import DeleteIngressRuleRequest
from .types.appengine import DeleteInstanceRequest
from .types.appengine import DeleteServiceRequest
from .types.appengine import DeleteVersionRequest
from .types.appengine import GetApplicationRequest
from .types.appengine import GetAuthorizedCertificateRequest
from .types.appengine import GetDomainMappingRequest
from .types.appengine import GetIngressRuleRequest
from .types.appengine import GetInstanceRequest
from .types.appengine import GetServiceRequest
from .types.appengine import GetVersionRequest
from .types.appengine import ListAuthorizedCertificatesRequest
from .types.appengine import ListAuthorizedCertificatesResponse
from .types.appengine import ListAuthorizedDomainsRequest
from .types.appengine import ListAuthorizedDomainsResponse
from .types.appengine import ListDomainMappingsRequest
from .types.appengine import ListDomainMappingsResponse
from .types.appengine import ListIngressRulesRequest
from .types.appengine import ListIngressRulesResponse
from .types.appengine import ListInstancesRequest
from .types.appengine import ListInstancesResponse
from .types.appengine import ListServicesRequest
from .types.appengine import ListServicesResponse
from .types.appengine import ListVersionsRequest
from .types.appengine import ListVersionsResponse
from .types.appengine import RepairApplicationRequest
from .types.appengine import UpdateApplicationRequest
from .types.appengine import UpdateAuthorizedCertificateRequest
from .types.appengine import UpdateDomainMappingRequest
from .types.appengine import UpdateIngressRuleRequest
from .types.appengine import UpdateServiceRequest
from .types.appengine import UpdateVersionRequest
from .types.appengine import AuthorizedCertificateView
from .types.appengine import DomainOverrideStrategy
from .types.appengine import VersionView
from .types.application import Application
from .types.application import UrlDispatchRule
from .types.audit_data import AuditData
from .types.audit_data import CreateVersionMethod
from .types.audit_data import UpdateServiceMethod
from .types.certificate import AuthorizedCertificate
from .types.certificate import CertificateRawData
from .types.certificate import ManagedCertificate
from .types.certificate import ManagementStatus
from .types.deploy import CloudBuildOptions
from .types.deploy import ContainerInfo
from .types.deploy import Deployment
from .types.deploy import FileInfo
from .types.deploy import ZipInfo
from .types.domain import AuthorizedDomain
from .types.domain_mapping import DomainMapping
from .types.domain_mapping import ResourceRecord
from .types.domain_mapping import SslSettings
from .types.firewall import FirewallRule
from .types.instance import Instance
from .types.location import LocationMetadata
from .types.network_settings import NetworkSettings
from .types.operation import CreateVersionMetadataV1
from .types.operation import OperationMetadataV1
from .types.service import Service
from .types.service import TrafficSplit
from .types.version import AutomaticScaling
from .types.version import BasicScaling
from .types.version import CpuUtilization
from .types.version import DiskUtilization
from .types.version import EndpointsApiService
from .types.version import Entrypoint
from .types.version import ManualScaling
from .types.version import Network
from .types.version import NetworkUtilization
from .types.version import RequestUtilization
from .types.version import Resources
from .types.version import StandardSchedulerSettings
from .types.version import Version
from .types.version import Volume
from .types.version import VpcAccessConnector
from .types.version import InboundServiceType
from .types.version import ServingStatus

__all__ = (
    "ApplicationsAsyncClient",
    "AuthorizedCertificatesAsyncClient",
    "AuthorizedDomainsAsyncClient",
    "DomainMappingsAsyncClient",
    "FirewallAsyncClient",
    "InstancesAsyncClient",
    "ServicesAsyncClient",
    "VersionsAsyncClient",
    "ApiConfigHandler",
    "ApiEndpointHandler",
    "Application",
    "ApplicationsClient",
    "AuditData",
    "AuthFailAction",
    "AuthorizedCertificate",
    "AuthorizedCertificateView",
    "AuthorizedCertificatesClient",
    "AuthorizedDomain",
    "AuthorizedDomainsClient",
    "AutomaticScaling",
    "BasicScaling",
    "BatchUpdateIngressRulesRequest",
    "BatchUpdateIngressRulesResponse",
    "CertificateRawData",
    "CloudBuildOptions",
    "ContainerInfo",
    "CpuUtilization",
    "CreateApplicationRequest",
    "CreateAuthorizedCertificateRequest",
    "CreateDomainMappingRequest",
    "CreateIngressRuleRequest",
    "CreateVersionMetadataV1",
    "CreateVersionMethod",
    "CreateVersionRequest",
    "DebugInstanceRequest",
    "DeleteAuthorizedCertificateRequest",
    "DeleteDomainMappingRequest",
    "DeleteIngressRuleRequest",
    "DeleteInstanceRequest",
    "DeleteServiceRequest",
    "DeleteVersionRequest",
    "Deployment",
    "DiskUtilization",
    "DomainMapping",
    "DomainMappingsClient",
    "DomainOverrideStrategy",
    "EndpointsApiService",
    "Entrypoint",
    "ErrorHandler",
    "FileInfo",
    "FirewallClient",
    "FirewallRule",
    "GetApplicationRequest",
    "GetAuthorizedCertificateRequest",
    "GetDomainMappingRequest",
    "GetIngressRuleRequest",
    "GetInstanceRequest",
    "GetServiceRequest",
    "GetVersionRequest",
    "HealthCheck",
    "InboundServiceType",
    "Instance",
    "InstancesClient",
    "Library",
    "ListAuthorizedCertificatesRequest",
    "ListAuthorizedCertificatesResponse",
    "ListAuthorizedDomainsRequest",
    "ListAuthorizedDomainsResponse",
    "ListDomainMappingsRequest",
    "ListDomainMappingsResponse",
    "ListIngressRulesRequest",
    "ListIngressRulesResponse",
    "ListInstancesRequest",
    "ListInstancesResponse",
    "ListServicesRequest",
    "ListServicesResponse",
    "ListVersionsRequest",
    "ListVersionsResponse",
    "LivenessCheck",
    "LocationMetadata",
    "LoginRequirement",
    "ManagedCertificate",
    "ManagementStatus",
    "ManualScaling",
    "Network",
    "NetworkSettings",
    "NetworkUtilization",
    "OperationMetadataV1",
    "ReadinessCheck",
    "RepairApplicationRequest",
    "RequestUtilization",
    "ResourceRecord",
    "Resources",
    "ScriptHandler",
    "SecurityLevel",
    "Service",
    "ServicesClient",
    "ServingStatus",
    "SslSettings",
    "StandardSchedulerSettings",
    "StaticFilesHandler",
    "TrafficSplit",
    "UpdateApplicationRequest",
    "UpdateAuthorizedCertificateRequest",
    "UpdateDomainMappingRequest",
    "UpdateIngressRuleRequest",
    "UpdateServiceMethod",
    "UpdateServiceRequest",
    "UpdateVersionRequest",
    "UrlDispatchRule",
    "UrlMap",
    "Version",
    "VersionView",
    "VersionsClient",
    "Volume",
    "VpcAccessConnector",
    "ZipInfo",
)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.protobuf import duration_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.appengine.v1", manifest={"Application", "UrlDispatchRule",},
)


class Application(proto.Message):
    r"""An Application resource contains the top-level configuration
    of an App Engine application.

    Attributes:
        name (str):
            Full path to the Application resource in the API. Example:
            ``apps/myapp``.

            @OutputOnly
        id (str):
            Identifier of the Application resource. This identifier is
            equivalent to the project ID of the Google Cloud Platform
            project where you want to deploy your application. Example:
            ``myapp``.
        dispatch_rules (Sequence[google.cloud.appengine_admin_v1.types.UrlDispatchRule]):
            HTTP path dispatch rules for requests to the
            application that do not explicitly target a
            service or version. Rules are order-dependent.
            Up to 20 dispatch rules can be supported.
        auth_domain (str):
            Google Apps authentication domain that
            controls which users can access this
            application.
            Defaults to open access for any Google Account.
        location_id (str):
            Location from which this application runs. Application
            instances run out of the data centers in the specified
            location, which is also where all of the application's end
            user content is stored.

            Defaults to ``us-central``.

            View the list of `supported
            locations <https://cloud.google.com/appengine/docs/locations>`__.
        code_bucket (str):
            Google Cloud Storage bucket that can be used
            for storing files associated with this
            application. This bucket is associated with the
            application and can be used by the gcloud
            deployment commands.
            @OutputOnly
        default_cookie_expiration (google.protobuf.duration_pb2.Duration):
            Cookie expiration policy for this
            application.
        serving_status (google.cloud.appengine_admin_v1.types.Application.ServingStatus):
            Serving status of this application.
        default_hostname (str):
            Hostname used to reach this application, as
            resolved by App Engine.
            @OutputOnly
        default_bucket (str):
            Google Cloud Storage bucket that can be used
            by this application to store content.

            @OutputOnly
        iap (google.cloud.appengine_admin_v1.types.Application.IdentityAwareProxy):

        gcr_domain (str):
            The Google Container Registry domain used for
            storing managed build docker images for this
            application.
        database_type (google.cloud.appengine_admin_v1.types.Application.DatabaseType):
            The type of the Cloud Firestore or Cloud
            Datastore database associated with this
            application.
        feature_settings (google.cloud.appengine_admin_v1.types.Application.FeatureSettings):
            The feature specific settings to be used in
            the application.
    """

    class ServingStatus(proto.Enum):
        r""""""
        UNSPECIFIED = 0
        SERVING = 1
        USER_DISABLED = 2
        SYSTEM_DISABLED = 3

    class DatabaseType(proto.Enum):
        r""""""
        DATABASE_TYPE_UNSPECIFIED = 0
        CLOUD_DATASTORE = 1
        CLOUD_FIRESTORE = 2
        CLOUD_DATASTORE_COMPATIBILITY = 3

    class IdentityAwareProxy(proto.Message):
        r"""Identity-Aware Proxy
        Attributes:
            enabled (bool):
                Whether the serving infrastructure will authenticate and
                authorize all incoming requests.

                If true, the ``oauth2_client_id`` and
                ``oauth2_client_secret`` fields must be non-empty.
            oauth2_client_id (str):
                OAuth2 client ID to use for the
                authentication flow.
            oauth2_client_secret (str):
                OAuth2 client secret to use for the authentication flow.

                For security reasons, this value cannot be retrieved via the
                API. Instead, the SHA-256 hash of the value is returned in
                the ``oauth2_client_secret_sha256`` field.

                @InputOnly
            oauth2_client_secret_sha256 (str):
                Hex-encoded SHA-256 hash of the client
                secret.
                @OutputOnly
        """

        enabled = proto.Field(proto.BOOL, number=1,)
        oauth2_client_id = proto.Field(proto.STRING, number=2,)
        oauth2_client_secret = proto.Field(proto.STRING, number=3,)
        oauth2_client_secret_sha256 = proto.Field(proto.STRING, number=4,)

    class FeatureSettings(proto.Message):
        r"""The feature specific settings to be used in the application.
        These define behaviors that are user configurable.

        Attributes:
            split_health_checks (bool):
                Boolean value indicating if split health checks should be
                used instead of the legacy health checks. At an app.yaml
                level, this means defaulting to 'readiness_check' and
                'liveness_check' values instead of 'health_check' ones. Once
                the legacy 'health_check' behavior is deprecated, and this
                value is always true, this setting can be removed.
            use_container_optimized_os (bool):
                If true, use `Container-Optimized
                OS <https://cloud.google.com/container-optimized-os/>`__
                base image for VMs, rather than a base Debian image.
        """

        split_health_checks = proto.Field(proto.BOOL, number=1,)
        use_container_optimized_os = proto.Field(proto.BOOL, number=2,)

    name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.STRING, number=2,)
    dispatch_rules = proto.RepeatedField(
        proto.MESSAGE, number=3, message="UrlDispatchRule",
    )
    auth_domain = proto.Field(proto.STRING, number=6,)
    location_id = proto.Field(proto.STRING, number=7,)
    code_bucket = proto.Field(proto.STRING, number=8,)
    default_cookie_expiration = proto.Field(
        proto.MESSAGE, number=9, message=duration_pb2.Duration,
    )
    serving_status = proto.Field(proto.ENUM, number=10, enum=ServingStatus,)
    default_hostname = proto.Field(proto.STRING, number=11,)
    default_bucket = proto.Field(proto.STRING, number=12,)
    iap = proto.Field(proto.MESSAGE, number=14, message=IdentityAwareProxy,)
    gcr_domain = proto.Field(proto.STRING, number=16,)
    database_type = proto.Field(proto.ENUM, number=17, enum=DatabaseType,)
    feature_settings = proto.Field(proto.MESSAGE, number=18, message=FeatureSettings,)


class UrlDispatchRule(proto.Message):
    r"""Rules to match an HTTP request and dispatch that request to a
    service.

    Attributes:
        domain (str):
            Domain name to match against. The wildcard "``*``" is
            supported if specified before a period: "``*.``".

            Defaults to matching all domains: "``*``".
        path (str):
            Pathname within the host. Must start with a "``/``". A
            single "``*``" can be included at the end of the path.

            The sum of the lengths of the domain and path may not exceed
            100 characters.
        service (str):
            Resource ID of a service in this application that should
            serve the matched request. The service must already exist.
            Example: ``default``.
    """

    domain = proto.Field(proto.STRING, number=1,)
    path = proto.Field(proto.STRING, number=2,)
    service = proto.Field(proto.STRING, number=3,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import Dict, Sequence, Tuple, Type, Union
import pkg_resources

import google.api_core.client_options as ClientOptions  # type: ignore
from google.api_core import exceptions as core_exceptions  # type: ignore
from google.api_core import gapic_v1  # type: ignore
from google.api_core import retry as retries  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.api_core import operation as gac_operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.cloud.appengine_admin_v1.services.instances import pagers
from google.cloud.appengine_admin_v1.types import appengine
from google.cloud.appengine_admin_v1.types import instance
from google.cloud.appengine_admin_v1.types import operation as ga_operation
from google.protobuf import empty_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from .transports.base import InstancesTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import InstancesGrpcAsyncIOTransport
from .client import InstancesClient


class InstancesAsyncClient:
    """Manages instances of a version."""

    _client: InstancesClient

    DEFAULT_ENDPOINT = InstancesClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = InstancesClient.DEFAULT_MTLS_ENDPOINT

    instance_path = staticmethod(InstancesClient.instance_path)
    parse_instance_path = staticmethod(InstancesClient.parse_instance_path)
    common_billing_account_path = staticmethod(
        InstancesClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        InstancesClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(InstancesClient.common_folder_path)
    parse_common_folder_path = staticmethod(InstancesClient.parse_common_folder_path)
    common_organization_path = staticmethod(InstancesClient.common_organization_path)
    parse_common_organization_path = staticmethod(
        InstancesClient.parse_common_organization_path
    )
    common_project_path = staticmethod(InstancesClient.common_project_path)
    parse_common_project_path = staticmethod(InstancesClient.parse_common_project_path)
    common_location_path = staticmethod(InstancesClient.common_location_path)
    parse_common_location_path = staticmethod(
        InstancesClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            InstancesAsyncClient: The constructed client.
        """
        return InstancesClient.from_service_account_info.__func__(InstancesAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            InstancesAsyncClient: The constructed client.
        """
        return InstancesClient.from_service_account_file.__func__(InstancesAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> InstancesTransport:
        """Returns the transport used by the client instance.

        Returns:
            InstancesTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(InstancesClient).get_transport_class, type(InstancesClient)
    )

    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials = None,
        transport: Union[str, InstancesTransport] = "grpc_asyncio",
        client_options: ClientOptions = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the instances client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.InstancesTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = InstancesClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def list_instances(
        self,
        request: appengine.ListInstancesRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListInstancesAsyncPager:
        r"""Lists the instances of a version.

        Tip: To aggregate details about instances over time, see the
        `Stackdriver Monitoring
        API <https://cloud.google.com/monitoring/api/ref_v3/rest/v3/projects.timeSeries/list>`__.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.ListInstancesRequest`):
                The request object. Request message for
                `Instances.ListInstances`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.appengine_admin_v1.services.instances.pagers.ListInstancesAsyncPager:
                Response message for Instances.ListInstances.

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        request = appengine.ListInstancesRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_instances,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListInstancesAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_instance(
        self,
        request: appengine.GetInstanceRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> instance.Instance:
        r"""Gets instance information.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.GetInstanceRequest`):
                The request object. Request message for
                `Instances.GetInstance`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.appengine_admin_v1.types.Instance:
                An Instance resource is the computing
                unit that App Engine uses to
                automatically scale an application.

        """
        # Create or coerce a protobuf request object.
        request = appengine.GetInstanceRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_instance,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def delete_instance(
        self,
        request: appengine.DeleteInstanceRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Stops a running instance.

        The instance might be automatically recreated based on the
        scaling settings of the version. For more information, see "How
        Instances are Managed" (`standard
        environment <https://cloud.google.com/appengine/docs/standard/python/how-instances-are-managed>`__
        \| `flexible
        environment <https://cloud.google.com/appengine/docs/flexible/python/how-instances-are-managed>`__).

        To ensure that instances are not re-created and avoid getting
        billed, you can stop all instances within the target version by
        changing the serving status of the version to ``STOPPED`` with
        the
        ```apps.services.versions.patch`` <https://cloud.google.com/appengine/docs/admin-api/reference/rest/v1/apps.services.versions/patch>`__
        method.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.DeleteInstanceRequest`):
                The request object. Request message for
                `Instances.DeleteInstance`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

                   The JSON representation for Empty is empty JSON
                   object {}.

        """
        # Create or coerce a protobuf request object.
        request = appengine.DeleteInstanceRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_instance,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=ga_operation.OperationMetadataV1,
        )

        # Done; return the response.
        return response

    async def debug_instance(
        self,
        request: appengine.DebugInstanceRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Enables debugging on a VM instance. This allows you
        to use the SSH command to connect to the virtual machine
        where the instance lives. While in "debug mode", the
        instance continues to serve live traffic. You should
        delete the instance when you are done debugging and then
        allow the system to take over and determine if another
        instance should be started.

        Only applicable for instances in App Engine flexible
        environment.

        Args:
            request (:class:`google.cloud.appengine_admin_v1.types.DebugInstanceRequest`):
                The request object. Request message for
                `Instances.DebugInstance`.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.appengine_admin_v1.types.Instance` An Instance resource is the computing unit that App Engine uses to
                   automatically scale an application.

        """
        # Create or coerce a protobuf request object.
        request = appengine.DebugInstanceRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.debug_instance,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            instance.Instance,
            metadata_type=ga_operation.OperationMetadataV1,
        )

        # Done; return the response.
        return response


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-appengine-admin",
        ).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("InstancesAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .app_yaml import (
    ApiConfigHandler,
    ApiEndpointHandler,
    ErrorHandler,
    HealthCheck,
    Library,
    LivenessCheck,
    ReadinessCheck,
    ScriptHandler,
    StaticFilesHandler,
    UrlMap,
    AuthFailAction,
    LoginRequirement,
    SecurityLevel,
)
from .appengine import (
    BatchUpdateIngressRulesRequest,
    BatchUpdateIngressRulesResponse,
    CreateApplicationRequest,
    CreateAuthorizedCertificateRequest,
    CreateDomainMappingRequest,
    CreateIngressRuleRequest,
    CreateVersionRequest,
    DebugInstanceRequest,
    DeleteAuthorizedCertificateRequest,
    DeleteDomainMappingRequest,
    DeleteIngressRuleRequest,
    DeleteInstanceRequest,
    DeleteServiceRequest,
    DeleteVersionRequest,
    GetApplicationRequest,
    GetAuthorizedCertificateRequest,
    GetDomainMappingRequest,
    GetIngressRuleRequest,
    GetInstanceRequest,
    GetServiceRequest,
    GetVersionRequest,
    ListAuthorizedCertificatesRequest,
    ListAuthorizedCertificatesResponse,
    ListAuthorizedDomainsRequest,
    ListAuthorizedDomainsResponse,
    ListDomainMappingsRequest,
    ListDomainMappingsResponse,
    ListIngressRulesRequest,
    ListIngressRulesResponse,
    ListInstancesRequest,
    ListInstancesResponse,
    ListServicesRequest,
    ListServicesResponse,
    ListVersionsRequest,
    ListVersionsResponse,
    RepairApplicationRequest,
    UpdateApplicationRequest,
    UpdateAuthorizedCertificateRequest,
    UpdateDomainMappingRequest,
    UpdateIngressRuleRequest,
    UpdateServiceRequest,
    UpdateVersionRequest,
    AuthorizedCertificateView,
    DomainOverrideStrategy,
    VersionView,
)
from .application import (
    Application,
    UrlDispatchRule,
)
from .audit_data import (
    AuditData,
    CreateVersionMethod,
    UpdateServiceMethod,
)
from .certificate import (
    AuthorizedCertificate,
    CertificateRawData,
    ManagedCertificate,
    ManagementStatus,
)
from .deploy import (
    CloudBuildOptions,
    ContainerInfo,
    Deployment,
    FileInfo,
    ZipInfo,
)
from .domain import AuthorizedDomain
from .domain_mapping import (
    DomainMapping,
    ResourceRecord,
    SslSettings,
)
from .firewall import FirewallRule
from .instance import Instance
from .location import LocationMetadata
from .network_settings import NetworkSettings
from .operation import (
    CreateVersionMetadataV1,
    OperationMetadataV1,
)
from .service import (
    Service,
    TrafficSplit,
)
from .version import (
    AutomaticScaling,
    BasicScaling,
    CpuUtilization,
    DiskUtilization,
    EndpointsApiService,
    Entrypoint,
    ManualScaling,
    Network,
    NetworkUtilization,
    RequestUtilization,
    Resources,
    StandardSchedulerSettings,
    Version,
    Volume,
    VpcAccessConnector,
    InboundServiceType,
    ServingStatus,
)

__all__ = (
    "ApiConfigHandler",
    "ApiEndpointHandler",
    "ErrorHandler",
    "HealthCheck",
    "Library",
    "LivenessCheck",
    "ReadinessCheck",
    "ScriptHandler",
    "StaticFilesHandler",
    "UrlMap",
    "AuthFailAction",
    "LoginRequirement",
    "SecurityLevel",
    "BatchUpdateIngressRulesRequest",
    "BatchUpdateIngressRulesResponse",
    "CreateApplicationRequest",
    "CreateAuthorizedCertificateRequest",
    "CreateDomainMappingRequest",
    "CreateIngressRuleRequest",
    "CreateVersionRequest",
    "DebugInstanceRequest",
    "DeleteAuthorizedCertificateRequest",
    "DeleteDomainMappingRequest",
    "DeleteIngressRuleRequest",
    "DeleteInstanceRequest",
    "DeleteServiceRequest",
    "DeleteVersionRequest",
    "GetApplicationRequest",
    "GetAuthorizedCertificateRequest",
    "GetDomainMappingRequest",
    "GetIngressRuleRequest",
    "GetInstanceRequest",
    "GetServiceRequest",
    "GetVersionRequest",
    "ListAuthorizedCertificatesRequest",
    "ListAuthorizedCertificatesResponse",
    "ListAuthorizedDomainsRequest",
    "ListAuthorizedDomainsResponse",
    "ListDomainMappingsRequest",
    "ListDomainMappingsResponse",
    "ListIngressRulesRequest",
    "ListIngressRulesResponse",
    "ListInstancesRequest",
    "ListInstancesResponse",
    "ListServicesRequest",
    "ListServicesResponse",
    "ListVersionsRequest",
    "ListVersionsResponse",
    "RepairApplicationRequest",
    "UpdateApplicationRequest",
    "UpdateAuthorizedCertificateRequest",
    "UpdateDomainMappingRequest",
    "UpdateIngressRuleRequest",
    "UpdateServiceRequest",
    "UpdateVersionRequest",
    "AuthorizedCertificateView",
    "DomainOverrideStrategy",
    "VersionView",
    "Application",
    "UrlDispatchRule",
    "AuditData",
    "CreateVersionMethod",
    "UpdateServiceMethod",
    "AuthorizedCertificate",
    "CertificateRawData",
    "ManagedCertificate",
    "ManagementStatus",
    "CloudBuildOptions",
    "ContainerInfo",
    "Deployment",
    "FileInfo",
    "ZipInfo",
    "AuthorizedDomain",
    "DomainMapping",
    "ResourceRecord",
    "SslSettings",
    "FirewallRule",
    "Instance",
    "LocationMetadata",
    "NetworkSettings",
    "CreateVersionMetadataV1",
    "OperationMetadataV1",
    "Service",
    "TrafficSplit",
    "AutomaticScaling",
    "BasicScaling",
    "CpuUtilization",
    "DiskUtilization",
    "EndpointsApiService",
    "Entrypoint",
    "ManualScaling",
    "Network",
    "NetworkUtilization",
    "RequestUtilization",
    "Resources",
    "StandardSchedulerSettings",
    "Version",
    "Volume",
    "VpcAccessConnector",
    "InboundServiceType",
    "ServingStatus",
)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.appengine_admin_v1.types import appengine


__protobuf__ = proto.module(
    package="google.appengine.v1",
    manifest={"AuditData", "UpdateServiceMethod", "CreateVersionMethod",},
)


class AuditData(proto.Message):
    r"""App Engine admin service audit log.
    Attributes:
        update_service (google.cloud.appengine_admin_v1.types.UpdateServiceMethod):
            Detailed information about UpdateService
            call.
        create_version (google.cloud.appengine_admin_v1.types.CreateVersionMethod):
            Detailed information about CreateVersion
            call.
    """

    update_service = proto.Field(
        proto.MESSAGE, number=1, oneof="method", message="UpdateServiceMethod",
    )
    create_version = proto.Field(
        proto.MESSAGE, number=2, oneof="method", message="CreateVersionMethod",
    )


class UpdateServiceMethod(proto.Message):
    r"""Detailed information about UpdateService call.
    Attributes:
        request (google.cloud.appengine_admin_v1.types.UpdateServiceRequest):
            Update service request.
    """

    request = proto.Field(
        proto.MESSAGE, number=1, message=appengine.UpdateServiceRequest,
    )


class CreateVersionMethod(proto.Message):
    r"""Detailed information about CreateVersion call.
    Attributes:
        request (google.cloud.appengine_admin_v1.types.CreateVersionRequest):
            Create version request.
    """

    request = proto.Field(
        proto.MESSAGE, number=1, message=appengine.CreateVersionRequest,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
